# Copyright (C) 2011  Chris Lalancette <clalance@redhat.com>
# Copyright (C) 2012-2017  Chris Lalancette <clalancette@gmail.com>

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation;
# version 2.1 of the License.

# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

"""
Mandriva installation
"""

import os
import re
import shutil

import oz.Guest
import oz.OzException
import oz.ozutil


class MandrivaConfiguration(object):
    """
    Configuration class for Mandriva.
    """
    def __init__(self, old_path):
        self._old_path = old_path

    @property
    def old_path(self):
        """
        Property method for whether this uses the old style paths.
        """
        return self._old_path


version_to_config = {
    "2008.0": MandrivaConfiguration(old_path=False),
    "2007.0": MandrivaConfiguration(old_path=False),
    "2006.0": MandrivaConfiguration(old_path=True),
    "2005": MandrivaConfiguration(old_path=True),
}


class MandrivaGuest(oz.Guest.CDGuest):
    """
    Class for Mandriva 2005, 2006.0, 2007.0, and 2008.0 installation.
    """
    def __init__(self, tdl, config, auto, output_disk, netdev, diskbus,
                 macaddress):
        oz.Guest.CDGuest.__init__(self, tdl, config, auto, output_disk, netdev,
                                  None, None, diskbus, True, False, macaddress)

        self.mandriva_arch = self.tdl.arch
        if self.mandriva_arch == "i386":
            self.mandriva_arch = "i586"

        self.config = version_to_config[tdl.update]

    def _modify_iso(self):
        """
        Method to make the boot ISO auto-boot with appropriate parameters.
        """
        self.log.debug("Modifying ISO")

        self.log.debug("Copying cfg file")

        pathdir = self.iso_contents
        if not self.config.old_path:
            pathdir = os.path.join(self.iso_contents, self.mandriva_arch)

        outname = os.path.join(pathdir, "auto_inst.cfg")

        if self.default_auto_file():

            def _cfg_sub(line):
                """
                Method that is called back from oz.ozutil.copy_modify_file() to
                modify preseed files as appropriate for Mandriva.
                """
                if re.search("'password' =>", line):
                    return "			'password' => '" + self.rootpw + "',\n"
                return line

            oz.ozutil.copy_modify_file(self.auto, outname, _cfg_sub)
        else:
            shutil.copy(self.auto, outname)

        self.log.debug("Modifying isolinux.cfg")
        isolinuxcfg = os.path.join(pathdir, "isolinux", "isolinux.cfg")
        with open(isolinuxcfg, 'w') as f:
            f.write("""\
default customiso
timeout 1
prompt 0
label customiso
  kernel alt0/vmlinuz
  append initrd=alt0/all.rdz ramdisk_size=128000 root=/dev/ram3 acpi=ht vga=788 automatic=method:cdrom kickstart=auto_inst.cfg
""")

    def _generate_new_iso(self):
        """
        Method to create a new ISO based on the modified CD/DVD.
        """
        self.log.info("Generating new ISO")

        isolinuxdir = ""
        if not self.config.old_path:
            isolinuxdir = self.mandriva_arch

        isolinuxbin = os.path.join(isolinuxdir, "isolinux/isolinux.bin")
        isolinuxboot = os.path.join(isolinuxdir, "isolinux/boot.cat")

        oz.ozutil.subprocess_check_output(["genisoimage", "-r", "-V", "Custom",
                                           "-J", "-l", "-no-emul-boot",
                                           "-b", isolinuxbin,
                                           "-c", isolinuxboot,
                                           "-boot-load-size", "4",
                                           "-cache-inodes", "-boot-info-table",
                                           "-v", "-o", self.output_iso,
                                           self.iso_contents],
                                          printfn=self.log.debug)


def get_class(tdl, config, auto, output_disk=None, netdev=None, diskbus=None,
              macaddress=None):
    """
    Factory method for Mandriva installs.
    """
    if tdl.update in version_to_config.keys():
        return MandrivaGuest(tdl, config, auto, output_disk, netdev, diskbus,
                             macaddress)


def get_supported_string():
    """
    Return supported versions as a string.
    """
    return "Mandriva: " + ", ".join(sorted(version_to_config.keys()))



# Copyright (C) 2010,2011  Chris Lalancette <clalance@redhat.com>
# Copyright (C) 2012-2017  Chris Lalancette <clalancette@gmail.com>

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation;
# version 2.1 of the License.

# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

"""
Common methods for installing and configuring RedHat-based guests
"""

try:
    import configparser
except ImportError:
    import ConfigParser as configparser
import gzip
import os
import re
import shutil

import oz.Guest
import oz.GuestFSManager
import oz.Linux
import oz.OzException
import oz.ozutil


class RedHatLinuxCDGuest(oz.Linux.LinuxCDGuest):
    """
    Class for RedHat-based CD guests.
    """
    def __init__(self, tdl, config, auto, output_disk, nicmodel, diskbus,
                 iso_allowed, url_allowed, initrdtype, macaddress):
        oz.Linux.LinuxCDGuest.__init__(self, tdl, config, auto, output_disk,
                                       nicmodel, diskbus, iso_allowed,
                                       url_allowed, macaddress)
        self.crond_was_active = False
        self.sshd_was_active = False
        self.sshd_config = """\
SyslogFacility AUTHPRIV
PasswordAuthentication yes
ChallengeResponseAuthentication no
GSSAPIAuthentication yes
GSSAPICleanupCredentials yes
UsePAM yes
AcceptEnv LANG LC_CTYPE LC_NUMERIC LC_TIME LC_COLLATE LC_MONETARY LC_MESSAGES
AcceptEnv LC_PAPER LC_NAME LC_ADDRESS LC_TELEPHONE LC_MEASUREMENT
AcceptEnv LC_IDENTIFICATION LC_ALL LANGUAGE
AcceptEnv XMODIFIERS
X11Forwarding yes
Subsystem	sftp	/usr/libexec/openssh/sftp-server
"""

        # initrdtype is actually a tri-state:
        # None - don't try to do direct kernel/initrd boot
        # "cpio" - Attempt to do direct kernel/initrd boot with a gzipped CPIO
        #        archive
        # "ext2" - Attempt to do direct kernel/initrd boot with a gzipped ext2
        #         filesystem
        self.initrdtype = initrdtype

        self.kernelfname = os.path.join(self.output_dir,
                                        self.tdl.name + "-kernel")
        self.initrdfname = os.path.join(self.output_dir,
                                        self.tdl.name + "-ramdisk")
        self.kernelcache = os.path.join(self.data_dir, "kernels",
                                        self.tdl.distro + self.tdl.update + self.tdl.arch + "-kernel")
        self.initrdcache = os.path.join(self.data_dir, "kernels",
                                        self.tdl.distro + self.tdl.update + self.tdl.arch + "-ramdisk")

        self.cmdline = "method=" + self.url + " ks=file:/ks.cfg"
        if self.tdl.kernel_param:
            self.cmdline += " " + self.tdl.kernel_param

        self.virtio_channel_name = None

    def _generate_new_iso(self):
        """
        Method to create a new ISO based on the modified CD/DVD.
        """
        self.log.debug("Generating new ISO")
        oz.ozutil.subprocess_check_output(["genisoimage", "-r", "-T", "-J",
                                           "-V", "Custom", "-no-emul-boot",
                                           "-b", "isolinux/isolinux.bin",
                                           "-c", "isolinux/boot.cat",
                                           "-boot-load-size", "4",
                                           "-boot-info-table", "-v",
                                           "-o", self.output_iso,
                                           self.iso_contents],
                                          printfn=self.log.debug)

    def _check_iso_tree(self, customize_or_icicle):
        kernel = os.path.join(self.iso_contents, "isolinux", "vmlinuz")
        if not os.path.exists(kernel):
            raise oz.OzException.OzException("Fedora/Red Hat installs can only be done using a boot.iso (netinst) or DVD image (LiveCDs are not supported)")

    def _modify_isolinux(self, initrdline):
        """
        Method to modify the isolinux.cfg file on a RedHat style CD.
        """
        self.log.debug("Modifying isolinux.cfg")
        # append additional kernel params from the TDL to initrdline
        if self.tdl.kernel_param:
            initrdline += " " + self.tdl.kernel_param
        initrdline += '\n'
        isolinuxcfg = os.path.join(self.iso_contents, "isolinux",
                                   "isolinux.cfg")

        with open(isolinuxcfg, "w") as f:
            f.write("""\
default customiso
timeout 1
prompt 0
label customiso
  kernel vmlinuz
%s
""" % (initrdline))

    def _copy_kickstart(self, outname):
        """
        Method to copy and modify a RedHat style kickstart file.
        """
        self.log.debug("Putting the kickstart in place")

        if self.default_auto_file():
            def _kssub(line):
                """
                Method that is called back from oz.ozutil.copy_modify_file() to
                modify kickstart files as appropriate.
                """
                if re.match("^rootpw", line):
                    return "rootpw " + self.rootpw + '\n'
                return line

            oz.ozutil.copy_modify_file(self.auto, outname, _kssub)
        else:
            shutil.copy(self.auto, outname)

    def _get_service_runlevel_link(self, g_handle, service):
        """
        Method to find the runlevel link(s) for a service based on the name
        and the (detected) default runlevel.
        """
        runlevel = self.get_default_runlevel(g_handle)

        lines = g_handle.cat('/etc/init.d/' + service).split("\n")
        startlevel = "99"
        for line in lines:
            if re.match('# chkconfig:', line):
                try:
                    startlevel = line.split(':')[1].split()[1]
                except Exception:
                    pass
                break

        return "/etc/rc.d/rc" + runlevel + ".d/S" + startlevel + service

    def _image_ssh_teardown_step_1(self, g_handle):
        """
        First step to undo _image_ssh_setup (remove authorized keys).
        """
        self.log.debug("Teardown step 1")
        # reset the authorized keys
        self.log.debug("Resetting authorized_keys")
        g_handle.path_restore('/root/.ssh')

    def _image_ssh_teardown_step_2(self, g_handle):
        """
        Second step to undo _image_ssh_setup (reset sshd service).
        """
        self.log.debug("Teardown step 2")
        # remove custom sshd_config
        self.log.debug("Resetting sshd_config")
        g_handle.path_restore('/etc/ssh/sshd_config')

        # reset the service link
        self.log.debug("Resetting sshd service")
        if g_handle.exists('/lib/systemd/system/sshd.service'):
            if not self.sshd_was_active:
                g_handle.rm('/etc/systemd/system/multi-user.target.wants/sshd.service')
        else:
            startuplink = self._get_service_runlevel_link(g_handle, 'sshd')
            g_handle.path_restore(startuplink)

    def _image_ssh_teardown_step_3(self, g_handle):
        """
        Third step to undo _image_ssh_setup (reset iptables).
        """
        self.log.debug("Teardown step 3")
        # reset iptables
        self.log.debug("Resetting iptables rules")
        g_handle.path_restore('/etc/sysconfig/iptables')

    def _image_ssh_teardown_step_4(self, g_handle):
        """
        Fourth step to undo _image_ssh_setup (remove guest announcement).
        """
        self.log.debug("Teardown step 4")
        self.log.debug("Removing announcement to host")
        g_handle.remove_if_exists('/etc/NetworkManager/dispatcher.d/99-reportip')

        # remove announce cronjob
        self.log.debug("Resetting announcement to host")
        g_handle.remove_if_exists('/etc/cron.d/announce')

        # remove reportip
        self.log.debug("Removing reportip")
        g_handle.remove_if_exists('/root/reportip')

        # reset the service link
        self.log.debug("Resetting crond service")
        if g_handle.exists('/lib/systemd/system/crond.service'):
            if not self.crond_was_active:
                g_handle.rm('/etc/systemd/system/multi-user.target.wants/crond.service')
        else:
            startuplink = self._get_service_runlevel_link(g_handle, 'crond')
            g_handle.path_restore(startuplink)

    def _image_ssh_teardown_step_5(self, g_handle):
        """
        Fifth step to undo _image_ssh_setup (reset SELinux).
        """
        self.log.debug("Teardown step 5")
        g_handle.path_restore("/etc/selinux/config")

    def _image_ssh_teardown_step_6(self, g_handle):
        """
        Sixth step to undo changes by the operating system.  For instance,
        during first boot openssh generates ssh host keys and stores them
        in /etc/ssh.  Since this image might be cached later on, this method
        removes those keys.
        """
        for key in g_handle.glob_expand("/etc/ssh/*_key*"):
            g_handle.remove_if_exists(key)

        # Remove any lease files; this is so that subsequent boots don't try
        # to connect to a DHCP server that is on a totally different network
        for lease in g_handle.glob_expand("/var/lib/dhclient/*.leases"):
            g_handle.rm(lease)

        for lease in g_handle.glob_expand("/var/lib/NetworkManager/*.lease"):
            g_handle.rm(lease)

    def _collect_teardown(self, libvirt_xml):
        """
        Method to reverse the changes done in _collect_setup.
        """
        self.log.info("Collection Teardown")

        g_handle = oz.GuestFSManager.GuestFSLibvirtFactory(libvirt_xml, self.libvirt_conn)
        g_handle.mount_partitions()

        try:
            self._image_ssh_teardown_step_1(g_handle)

            self._image_ssh_teardown_step_2(g_handle)

            self._image_ssh_teardown_step_3(g_handle)

            self._image_ssh_teardown_step_4(g_handle)

            self._image_ssh_teardown_step_5(g_handle)

            self._image_ssh_teardown_step_6(g_handle)
        finally:
            g_handle.cleanup()
            shutil.rmtree(self.icicle_tmp)

    def _image_ssh_setup_step_1(self, g_handle):
        """
        First step for allowing remote access (generate and upload ssh keys).
        """
        # part 1; upload the keys
        self.log.debug("Step 1: Uploading ssh keys")
        g_handle.path_backup('/root/.ssh')
        g_handle.mkdir('/root/.ssh')

        g_handle.path_backup('/root/.ssh/authorized_keys')

        self._generate_openssh_key(self.sshprivkey)

        g_handle.upload(self.sshprivkey + ".pub", '/root/.ssh/authorized_keys')

    def _image_ssh_setup_step_2(self, g_handle):
        """
        Second step for allowing remote access (configure sshd).
        """
        # part 2; check and setup sshd
        self.log.debug("Step 2: setup sshd")
        if not g_handle.exists('/usr/sbin/sshd'):
            raise oz.OzException.OzException("ssh not installed on the image, cannot continue")

        if g_handle.exists('/lib/systemd/system/sshd.service'):
            if g_handle.exists('/etc/systemd/system/multi-user.target.wants/sshd.service'):
                self.sshd_was_active = True
            else:
                g_handle.ln_sf('/lib/systemd/system/sshd.service',
                               '/etc/systemd/system/multi-user.target.wants/sshd.service')
        else:
            startuplink = self._get_service_runlevel_link(g_handle, 'sshd')
            g_handle.path_backup(startuplink)
            g_handle.ln_sf('/etc/init.d/sshd', startuplink)

        sshd_config_file = os.path.join(self.icicle_tmp, "sshd_config")
        with open(sshd_config_file, 'w') as f:
            f.write(self.sshd_config)

        try:
            g_handle.path_backup('/etc/ssh/sshd_config')
            g_handle.upload(sshd_config_file, '/etc/ssh/sshd_config')
        finally:
            os.unlink(sshd_config_file)

    def _image_ssh_setup_step_3(self, g_handle):
        """
        Third step for allowing remote access (open up the firewall).
        """
        # part 3; open up iptables
        self.log.debug("Step 3: Open up the firewall")
        g_handle.path_backup('/etc/sysconfig/iptables')

    def _image_ssh_setup_step_4(self, g_handle):
        """
        Fourth step for allowing remote access (make the guest announce itself
        on bootup).
        """
        # part 4; make sure the guest announces itself
        self.log.debug("Step 4: Guest announcement")

        if self.tdl.arch in ['ppc64', 'ppc64le']:
            announce_device = '/dev/hvc1'
        elif self.tdl.arch == 's390x':
            announce_device = '/dev/sclp_line0'
        else:
            announce_device = '/dev/ttyS1'

        scriptfile = os.path.join(self.icicle_tmp, "script")

        if g_handle.exists("/etc/NetworkManager/dispatcher.d"):
            with open(scriptfile, 'w') as f:
                f.write("""\
#!/bin/bash

if [ "$1" != "lo" -a "$2" = "up" ]; then
    echo -n "!$DHCP4_IP_ADDRESS,%s!" > %s
fi
""" % (self.uuid, announce_device))

            try:
                g_handle.upload(scriptfile,
                                '/etc/NetworkManager/dispatcher.d/99-reportip')
                g_handle.chmod(0o755,
                               '/etc/NetworkManager/dispatcher.d/99-reportip')
            finally:
                os.unlink(scriptfile)

        if not g_handle.exists('/usr/sbin/crond'):
            raise oz.OzException.OzException("cron not installed on the image, cannot continue")

        with open(scriptfile, 'w') as f:
            f.write("""\
#!/bin/bash
DEV=$(/bin/awk '{if ($2 == 0) print $1}' /proc/net/route) &&
[ -z "$DEV" ] && exit 0
ADDR=$(/sbin/ip -4 -o addr show dev $DEV | /bin/awk '{print $4}' | /bin/cut -d/ -f1) &&
[ -z "$ADDR" ] && exit 0
echo -n "!$ADDR,%s!" > %s
""" % (self.uuid, announce_device))

        try:
            g_handle.upload(scriptfile, '/root/reportip')
            g_handle.chmod(0o755, '/root/reportip')
        finally:
            os.unlink(scriptfile)

        announcefile = os.path.join(self.icicle_tmp, "announce")
        with open(announcefile, 'w') as f:
            f.write('*/1 * * * * root /bin/bash -c "/root/reportip"\n')

        try:
            g_handle.upload(announcefile, '/etc/cron.d/announce')
        finally:
            os.unlink(announcefile)

        if g_handle.exists('/lib/systemd/system/crond.service'):
            if g_handle.exists('/etc/systemd/system/multi-user.target.wants/crond.service'):
                self.crond_was_active = True
            else:
                g_handle.ln_sf('/lib/systemd/system/crond.service',
                               '/etc/systemd/system/multi-user.target.wants/crond.service')
        else:
            startuplink = self._get_service_runlevel_link(g_handle, 'crond')
            g_handle.path_backup(startuplink)
            g_handle.ln_sf('/etc/init.d/crond', startuplink)

    def _image_ssh_setup_step_5(self, g_handle):
        """
        Fifth step for allowing remote access (set SELinux to permissive).
        """
        # part 5; set SELinux to permissive mode so we don't have to deal with
        # incorrect contexts
        self.log.debug("Step 5: Set SELinux to permissive mode")
        g_handle.path_backup('/etc/selinux/config')

        selinuxfile = self.icicle_tmp + "/selinux"
        with open(selinuxfile, 'w') as f:
            f.write("SELINUX=permissive\n")
            f.write("SELINUXTYPE=targeted\n")

        try:
            g_handle.upload(selinuxfile, "/etc/selinux/config")
        finally:
            os.unlink(selinuxfile)

    def _collect_setup(self, libvirt_xml):
        """
        Setup the guest for remote access.
        """
        self.log.info("Collection Setup")

        g_handle = oz.GuestFSManager.GuestFSLibvirtFactory(libvirt_xml, self.libvirt_conn)
        g_handle.mount_partitions()

        # we have to do 5 things to make sure we can ssh into RHEL/Fedora:
        # 1)  Upload our ssh key
        # 2)  Make sure sshd is running on boot
        # 3)  Make sure that port 22 is open in the firewall
        # 4)  Make the guest announce itself to the host
        # 5)  Set SELinux to permissive mode

        try:
            try:
                self._image_ssh_setup_step_1(g_handle)

                try:
                    self._image_ssh_setup_step_2(g_handle)

                    try:
                        self._image_ssh_setup_step_3(g_handle)

                        try:
                            self._image_ssh_setup_step_4(g_handle)

                            try:
                                self._image_ssh_setup_step_5(g_handle)
                            except:
                                self._image_ssh_teardown_step_5(g_handle)
                                raise
                        except:
                            self._image_ssh_teardown_step_4(g_handle)
                            raise
                    except:
                        self._image_ssh_teardown_step_3(g_handle)
                        raise
                except:
                    self._image_ssh_teardown_step_2(g_handle)
                    raise
            except:
                self._image_ssh_teardown_step_1(g_handle)
                raise

        finally:
            g_handle.cleanup()

    def do_icicle(self, guestaddr):
        """
        Method to collect the package information and generate the ICICLE
        XML.
        """
        self.log.debug("Generating ICICLE")
        stdout, stderr_unused, retcode_unused = self.guest_execute_command(guestaddr,
                                                                           'rpm -qa',
                                                                           timeout=30)

        package_split = stdout.split("\n")

        extrasplit = None
        if self.tdl.icicle_extra_cmd:
            extrastdout, stderr_unused, retcode_unused = self.guest_execute_command(guestaddr,
                                                                                    self.tdl.icicle_extra_cmd,
                                                                                    timeout=30)
            extrasplit = extrastdout.split("\n")

            if len(package_split) != len(extrasplit):
                raise oz.OzException.OzException("Invalid extra package command; it must return the same set of packages as 'rpm -qa'")

        return self._output_icicle_xml(package_split, self.tdl.description,
                                       extrasplit)

    def _parse_treeinfo(self, fp):
        """
        Internal method to parse the treeinfo file and get the kernel and
        initrd paths out of it.
        """
        self.log.debug("Got treeinfo, parsing")
        config = configparser.SafeConfigParser()
        config.readfp(fp)
        section = "images-%s" % (self.tdl.arch)
        kernel = oz.ozutil.config_get_key(config, section, "kernel", None)
        initrd = oz.ozutil.config_get_key(config, section, "initrd", None)
        return (kernel, initrd)

    def _get_kernel_from_treeinfo(self, fetchurl):
        """
        Internal method to download and parse the .treeinfo file from a URL.  If
        the .treeinfo file does not exist, or it does not have the keys that we
        expect, this method raises an error.
        """
        treeinfourl = fetchurl + "/.treeinfo"

        # first we check if the .treeinfo exists; this throws an exception if
        # it is missing
        info = oz.ozutil.http_get_header(treeinfourl)
        if info['HTTP-Code'] != 200:
            raise oz.OzException.OzException("Could not find %s" % (treeinfourl))

        treeinfo = os.path.join(self.icicle_tmp, "treeinfo")
        self.log.debug("Going to write treeinfo to %s", treeinfo)
        treeinfofd = os.open(treeinfo, os.O_RDWR | os.O_CREAT | os.O_TRUNC)
        fp = os.fdopen(treeinfofd)
        try:
            os.unlink(treeinfo)
            self.log.debug("Trying to get treeinfo from " + treeinfourl)
            oz.ozutil.http_download_file(treeinfourl, treeinfofd,
                                         False, self.log)

            # if we made it here, the .treeinfo existed.  Parse it and
            # find out the location of the vmlinuz and initrd
            fp.seek(0)
            (kernel, initrd) = self._parse_treeinfo(fp)
        finally:
            fp.close()

        if kernel is None or initrd is None:
            raise oz.OzException.OzException("Empty kernel or initrd")

        self.log.debug("Returning kernel %s and initrd %s", kernel, initrd)
        return (kernel, initrd)

    def _create_cpio_initrd(self, kspath):
        """
        Internal method to create a modified CPIO initrd
        """
        # if initrdtype is cpio, then we can just append a gzipped
        # archive onto the end of the initrd
        extrafname = os.path.join(self.icicle_tmp, "extra.cpio")
        self.log.debug("Writing cpio to %s", extrafname)
        cpiofiledict = {}
        cpiofiledict[kspath] = 'ks.cfg'
        oz.ozutil.write_cpio(cpiofiledict, extrafname)

        try:
            shutil.copyfile(self.initrdcache, self.initrdfname)
            oz.ozutil.gzip_append(extrafname, self.initrdfname)
        finally:
            os.unlink(extrafname)

    def _create_ext2_initrd(self, kspath):
        """
        Internal method to create a modified ext2 initrd
        """
        # in this case, the archive is not CPIO but is an ext2
        # filesystem.  use guestfs to mount it and add the kickstart
        self.log.debug("Creating temporary directory")
        tmpdir = os.path.join(self.icicle_tmp, "initrd")
        oz.ozutil.mkdir_p(tmpdir)

        ext2file = os.path.join(tmpdir, "initrd.ext2")
        self.log.debug("Uncompressing initrd %s to %s", self.initrdfname, ext2file)
        inf = gzip.open(self.initrdcache, 'rb')
        outf = open(ext2file, "w")
        try:
            outf.writelines(inf)
            inf.close()

            g_handle = oz.GuestFSManager.GuestFS(ext2file, 'raw')
            g_handle.mount_partitions()
            g_handle.upload(kspath, "/ks.cfg")
            g_handle.cleanup()

            # kickstart is added, lets recompress it
            oz.ozutil.gzip_create(ext2file, self.initrdfname)
        finally:
            os.unlink(ext2file)

    def _initrd_inject_ks(self, fetchurl, force_download):
        """
        Internal method to download and inject a kickstart into an initrd.
        """
        # we first see if we can use direct kernel booting, as that is
        # faster than downloading the ISO
        kernel = None
        initrd = None
        try:
            (kernel, initrd) = self._get_kernel_from_treeinfo(fetchurl)
        except Exception:
            pass

        if kernel is None:
            self.log.debug("Kernel was None, trying images/pxeboot/vmlinuz")
            # we couldn't find the kernel in the treeinfo, so try a
            # hard-coded path
            kernel = "images/pxeboot/vmlinuz"
        if initrd is None:
            self.log.debug("Initrd was None, trying images/pxeboot/initrd.img")
            # we couldn't find the initrd in the treeinfo, so try a
            # hard-coded path
            initrd = "images/pxeboot/initrd.img"

        (fd, outdir) = oz.ozutil.open_locked_file(self.kernelcache)

        try:
            self._get_original_media('/'.join([self.url.rstrip('/'),
                                               kernel.lstrip('/')]),
                                     fd, outdir, force_download)

            # if we made it here, then we can copy the kernel into place
            shutil.copyfile(self.kernelcache, self.kernelfname)
        finally:
            os.close(fd)

        (fd, outdir) = oz.ozutil.open_locked_file(self.initrdcache)

        try:
            try:
                self._get_original_media('/'.join([self.url.rstrip('/'),
                                                   initrd.lstrip('/')]),
                                         fd, outdir, force_download)
            except:
                os.unlink(self.kernelfname)
                raise

            try:
                kspath = os.path.join(self.icicle_tmp, "ks.cfg")
                self._copy_kickstart(kspath)

                try:
                    if self.initrdtype == "cpio":
                        self._create_cpio_initrd(kspath)
                    elif self.initrdtype == "ext2":
                        self._create_ext2_initrd(kspath)
                    else:
                        raise oz.OzException.OzException("Invalid initrdtype, this is a programming error")
                finally:
                    os.unlink(kspath)
            except:
                os.unlink(self.kernelfname)
                raise
        finally:
            os.close(fd)

    def generate_install_media(self, force_download=False,
                               customize_or_icicle=False):
        """
        Method to generate the install media for RedHat based operating
        systems.  If force_download is False (the default), then the
        original media will only be fetched if it is not cached locally.  If
        force_download is True, then the original media will be downloaded
        regardless of whether it is cached locally.
        """
        fetchurl = self.url
        if self.tdl.installtype == 'url':
            # set the fetchurl up-front so that if the OS doesn't support
            # initrd injection, or the injection fails for some reason, we
            # fall back to the boot.iso
            fetchurl += "/images/boot.iso"

            if self.initrdtype is not None:
                self.log.debug("Installtype is URL, trying to do direct kernel boot")
                try:
                    return self._initrd_inject_ks(self.url, force_download)
                except Exception as err:
                    # if any of the above failed, we couldn't use the direct
                    # kernel/initrd build method.  Fall back to trying to fetch
                    # the boot.iso instead
                    self.log.debug("Could not do direct boot, fetching boot.iso instead (the following error message is useful for bug reports, but can be ignored)")
                    self.log.debug(err)

        return self._iso_generate_install_media(fetchurl, force_download,
                                                customize_or_icicle)

    def cleanup_install(self):
        """
        Method to cleanup any transient install data.
        """
        self.log.info("Cleaning up after install")

        for fname in [self.output_iso, self.initrdfname, self.kernelfname]:
            try:
                os.unlink(fname)
            except Exception:
                pass

        if not self.cache_original_media:
            for fname in [self.orig_iso, self.kernelcache, self.initrdcache]:
                try:
                    os.unlink(fname)
                except Exception:
                    pass

    def install(self, timeout=None, force=False):
        """
        Method to run the operating system installation.
        """
        return self._do_install(timeout, force, 0, self.kernelfname,
                                self.initrdfname, self.cmdline, None,
                                self.virtio_channel_name)


class RedHatLinuxCDYumGuest(RedHatLinuxCDGuest):
    """
    Class for RedHat-based CD guests with yum support.
    """
    def __init__(self, tdl, config, auto, output_disk, nicmodel, diskbus,
                 iso_allowed, url_allowed, initrdtype, macaddress, use_yum):
        oz.RedHat.RedHatLinuxCDGuest.__init__(self, tdl, config, auto,
                                              output_disk, nicmodel, diskbus,
                                              iso_allowed, url_allowed,
                                              initrdtype, macaddress)

        self.use_yum = use_yum

    def _check_url(self, iso=True, url=True):
        """
        Method to check if a URL specified by the user is one that will work
        with anaconda.
        """
        url = RedHatLinuxCDGuest._check_url(self, iso, url)

        if self.tdl.installtype == 'url':
            # The HTTP/1.1 specification allows for servers that don't support
            # byte ranges; if the client requests it and the server doesn't
            # support it, it is supposed to return a header that looks like:
            #
            #   Accept-Ranges: none
            #
            # You can see this in action by editing httpd.conf:
            #
            #   ...
            #   LoadModule headers_module
            #   ...
            #   Header set Accept-Ranges "none"
            #
            # and then trying to fetch a file with wget:
            #
            #   wget --header="Range: bytes=5-10" http://path/to/my/file
            #
            # Unfortunately, anaconda does not honor this server header, and
            # blindly requests a byte range anyway.  When this happens, the
            # server throws a "403 Forbidden", and the URL install fails.
            #
            # There is the additional problem of mirror lists.  If we take
            # the original URL that was given to us, and it happens to be
            # a redirect, then what can happen is that each individual package
            # during the install can come from a different mirror (some of
            # which may not support the byte ranges).  To avoid both of these
            # problems, resolve the (possible) redirect to a real mirror, and
            # check if we hit a server that doesn't support ranges.  If we do
            # hit one of these, try up to 5 times to redirect to a different
            # mirror.  If after this we still cannot find a server that
            # supports byte ranges, fail.
            count = 5
            while count > 0:
                info = oz.ozutil.http_get_header(url, redirect=False)

                if 'Accept-Ranges' in info and info['Accept-Ranges'] == "none":
                    if url == info['Redirect-URL']:
                        # optimization; if the URL we resolved to is exactly
                        # the same as what we started with, this is *not*
                        # a redirect, and we should fail immediately
                        count = 0
                        break

                    count -= 1
                    continue

                if 'Redirect-URL' in info and info['Redirect-URL'] is not None:
                    url = info['Redirect-URL']
                break

            if count == 0:
                raise oz.OzException.OzException("%s URL installs cannot be done using servers that don't accept byte ranges.  Please try another mirror" % (self.tdl.distro))

        return url

    def _customize_repos(self, guestaddr):
        """
        Method to generate and upload custom repository files based on the TDL.
        """
        self.log.debug("Installing additional repository files")

        for repo in list(self.tdl.repositories.values()):
            filename = repo.name.replace(" ", "_") + ".repo"
            localname = os.path.join(self.icicle_tmp, filename)
            with open(localname, 'w') as f:
                f.write("[%s]\n" % repo.name.replace(" ", "_"))
                f.write("name=%s\n" % repo.name)
                f.write("baseurl=%s\n" % repo.url)
                f.write("skip_if_unavailable=1\n")
                f.write("enabled=1\n")

                if repo.sslverify:
                    f.write("sslverify=1\n")
                else:
                    f.write("sslverify=0\n")

                if repo.signed:
                    f.write("gpgcheck=1\n")
                else:
                    f.write("gpgcheck=0\n")

            try:
                remotename = os.path.join("/etc/yum.repos.d/", filename)
                self.guest_live_upload(guestaddr, localname, remotename)
            finally:
                os.unlink(localname)

    def _install_packages(self, guestaddr, packstr):
        if self.use_yum:
            self.guest_execute_command(guestaddr, 'yum -y install %s' % (packstr))
        else:
            self.guest_execute_command(guestaddr, 'dnf -y install %s' % (packstr))

    def _remove_repos(self, guestaddr):
        for repo in list(self.tdl.repositories.values()):
            if not repo.persisted:
                filename = os.path.join("/etc/yum.repos.d",
                                        repo.name.replace(" ", "_") + ".repo")
                self.guest_execute_command(guestaddr, "rm -f " + filename,
                                           timeout=30)


class RedHatFDGuest(oz.Guest.FDGuest):
    """
    Class for RedHat-based floppy guests.
    """
    def __init__(self, tdl, config, auto, output_disk, nicmodel, diskbus,
                 macaddress):
        oz.Guest.FDGuest.__init__(self, tdl, config, auto, output_disk,
                                  nicmodel, None, None, diskbus, macaddress)

        if self.tdl.arch != "i386":
            raise oz.OzException.OzException("Invalid arch " + self.tdl.arch + "for " + self.tdl.distro + " guest")

    def _modify_floppy(self):
        """
        Method to make the floppy auto-boot with appropriate parameters.
        """
        oz.ozutil.mkdir_p(self.floppy_contents)

        self.log.debug("Putting the kickstart in place")

        output_ks = os.path.join(self.floppy_contents, "ks.cfg")

        if self.default_auto_file():
            def _kssub(line):
                """
                Method that is called back from oz.ozutil.copy_modify_file() to
                modify kickstart files as appropriate for RHL.
                """
                if re.match("^url", line):
                    return "url --url " + self.url + "\n"
                elif re.match("^rootpw", line):
                    return "rootpw " + self.rootpw + "\n"
                return line

            oz.ozutil.copy_modify_file(self.auto, output_ks, _kssub)
        else:
            shutil.copy(self.auto, output_ks)

        oz.ozutil.subprocess_check_output(["mcopy", "-i", self.output_floppy,
                                           output_ks, "::KS.CFG"],
                                          printfn=self.log.debug)

        self.log.debug("Modifying the syslinux.cfg")

        syslinux = os.path.join(self.floppy_contents, "SYSLINUX.CFG")
        with open(syslinux, 'w') as outfile:
            outfile.write("""\
default customboot
prompt 1
timeout 1
label customboot
  kernel vmlinuz
  append initrd=initrd.img lang= devfs=nomount ramdisk_size=9126 ks=floppy method=%s
""" % (self.url))

        # sometimes, syslinux.cfg on the floppy gets marked read-only.  Avoid
        # problems with the subsequent mcopy by marking it read/write.
        oz.ozutil.subprocess_check_output(["mattrib", "-r", "-i",
                                           self.output_floppy,
                                           "::SYSLINUX.CFG"],
                                          printfn=self.log.debug)

        oz.ozutil.subprocess_check_output(["mcopy", "-n", "-o", "-i",
                                           self.output_floppy, syslinux,
                                           "::SYSLINUX.CFG"],
                                          printfn=self.log.debug)

    def generate_install_media(self, force_download=False,
                               customize_or_icicle=False):
        """
        Method to generate the install media for RedHat based operating
        systems that install from floppy.  If force_download is False (the
        default), then the original media will only be fetched if it is
        not cached locally.  If force_download is True, then the original
        media will be downloaded regardless of whether it is cached locally.
        """
        self.log.info("Generating install media")

        if not force_download:
            if os.access(self.jeos_filename, os.F_OK):
                # if we found a cached JEOS, we don't need to do anything here;
                # we'll copy the JEOS itself later on
                return
            elif os.access(self.modified_floppy_cache, os.F_OK):
                self.log.info("Using cached modified media")
                shutil.copyfile(self.modified_floppy_cache, self.output_floppy)
                return

        # name of the output file
        (fd, outdir) = oz.ozutil.open_locked_file(self.orig_floppy)

        try:
            self._get_original_floppy(self.url + "/images/bootnet.img", fd,
                                      outdir, force_download)
            self._copy_floppy()
            try:
                self._modify_floppy()
                if self.cache_modified_media:
                    self.log.info("Caching modified media for future use")
                    shutil.copyfile(self.output_floppy, self.modified_floppy_cache)
            finally:
                self._cleanup_floppy()
        finally:
            os.close(fd)



#!/usr/bin/python

import sys
import getpass
try:
    import configparser
except ImportError:
    import ConfigParser as configparser
try:
    from StringIO import StringIO
except:
    from io import StringIO
import logging
import os

# Find oz library
prefix = '.'
for i in range(0,3):
    if os.path.isdir(os.path.join(prefix, 'oz')):
        sys.path.insert(0, prefix)
        break
    else:
        prefix = '../' + prefix

try:
    import oz.TDL
    import oz.GuestFactory
except ImportError as e:
    print(e)
    print('Unable to import oz.  Is oz installed or in your PYTHONPATH?')
    sys.exit(1)

try:
    import py.test
except ImportError:
    print('Unable to import py.test.  Is py.test installed?')
    sys.exit(1)

def default_route():
    route_file = "/proc/net/route"
    d = open(route_file)

    defn = 0
    for line in d:
        info = line.split()
        if (len(info) != 11): # 11 = typical num of fields in the file
            logging.warn(_("Invalid line length while parsing %s.") %
                         (route_file))
            break
        try:
            route = int(info[1], 16)
            if route == 0:
                return info[0]
        except ValueError:
            continue
    raise Exception("Could not find default route")

# we find the default route for this machine.  Note that this very well
# may not be a bridge, but for the purposes of testing the factory, it
# doesn't really matter; it just has to have an IP address
route = default_route()

def setup_guest(xml, macaddress=None):
    tdl = oz.TDL.TDL(xml)

    config = configparser.SafeConfigParser()
    config.readfp(StringIO("[libvirt]\nuri=qemu:///session\nbridge_name=%s" % route))

    guest = oz.GuestFactory.guest_factory(tdl, config, None, macaddress=macaddress)
    return guest

tdlxml = """
<template>
  <name>tester</name>
  <os>
    <name>Fedora</name>
    <version>14</version>
    <arch>x86_64</arch>
    <install type='url'>
      <url>http://download.fedoraproject.org/pub/fedora/linux//releases/14/Fedora/x86_64/os/</url>
    </install>
  </os>
</template>
"""

tdlxml2 = """
<template>
  <name>tester</name>
  <os>
    <name>Fedora</name>
    <version>14</version>
    <arch>x86_64</arch>
    <install type='url'>
      <url>http://download.fedoraproject.org/pub/fedora/linux//releases/14/Fedora/x86_64/os/</url>
    </install>
  </os>
  <disk>
    <size>20</size>
  </disk>
</template>
"""


def test_geteltorito_none_src():
    guest = setup_guest(tdlxml)

    with py.test.raises(oz.OzException.OzException):
        guest._geteltorito(None, None)

def test_geteltorito_none_dst(tmpdir):
    guest = setup_guest(tdlxml)

    src = os.path.join(str(tmpdir), 'src')
    open(src, 'w').write('src')

    with py.test.raises(oz.OzException.OzException):
        guest._geteltorito(src, None)

def test_geteltorito_short_pvd(tmpdir):
    guest = setup_guest(tdlxml)

    src = os.path.join(str(tmpdir), 'src')
    open(src, 'w').write('foo')

    dst = os.path.join(str(tmpdir), 'dst')

    with py.test.raises(Exception):
        guest._geteltorito(src, dst)

def test_geteltorito_bogus_pvd_desc(tmpdir):
    guest = setup_guest(tdlxml)

    src = os.path.join(str(tmpdir), 'src')
    fd = open(src, 'w')
    fd.seek(16*2048)
    fd.write('\0'*128)
    fd.close()

    dst = os.path.join(str(tmpdir), 'dst')

    with py.test.raises(oz.OzException.OzException):
        guest._geteltorito(src, dst)

def test_geteltorito_bogus_pvd_ident(tmpdir):
    guest = setup_guest(tdlxml)

    src = os.path.join(str(tmpdir), 'src')
    fd = open(src, 'w')
    fd.seek(16*2048)
    fd.write("\x01")
    fd.write('\0'*127)
    fd.close()

    dst = os.path.join(str(tmpdir), 'dst')

    with py.test.raises(oz.OzException.OzException):
        guest._geteltorito(src, dst)

def test_geteltorito_bogus_pvd_unused(tmpdir):
    guest = setup_guest(tdlxml)

    src = os.path.join(str(tmpdir), 'src')
    fd = open(src, 'w')
    fd.seek(16*2048)
    fd.write("\x01")
    fd.write("CD001")
    fd.write("\0x1")
    fd.write('\0'*127)
    fd.close()

    dst = os.path.join(str(tmpdir), 'dst')

    with py.test.raises(oz.OzException.OzException):
        guest._geteltorito(src, dst)

def test_geteltorito_bogus_pvd_unused2(tmpdir):
    guest = setup_guest(tdlxml)

    src = os.path.join(str(tmpdir), 'src')
    fd = open(src, 'w')
    fd.seek(16*2048)
    fd.write("\x01")
    fd.write("CD001")
    fd.write("\x01")
    fd.write("\x00")
    fd.write("AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA")
    fd.write("BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBB")
    fd.write("\x01")
    fd.write('\0'*127)
    fd.close()

    dst = os.path.join(str(tmpdir), 'dst')

    with py.test.raises(oz.OzException.OzException):
        guest._geteltorito(src, dst)

def test_geteltorito_short_boot_sector(tmpdir):
    guest = setup_guest(tdlxml)

    src = os.path.join(str(tmpdir), 'src')
    fd = open(src, 'w')
    fd.seek(16*2048)
    fd.write("\x01")
    fd.write("CD001")
    fd.write("\x01")
    fd.write("\x00")
    fd.write("AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA")
    fd.write("BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBB")
    fd.write("\x00")
    fd.write('\0'*127)
    fd.close()

    dst = os.path.join(str(tmpdir), 'dst')

    with py.test.raises(Exception):
        guest._geteltorito(src, dst)

def test_geteltorito_bogus_boot_sector(tmpdir):
    guest = setup_guest(tdlxml)

    src = os.path.join(str(tmpdir), 'src')
    fd = open(src, 'w')
    fd.seek(16*2048)
    fd.write("\x01")
    fd.write("CD001")
    fd.write("\x01")
    fd.write("\x00")
    fd.write("AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA")
    fd.write("BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBB")
    fd.write("\x00")
    fd.write('\0'*127)
    fd.seek(17*2048)
    fd.write("\x01")
    fd.write('\0'*75)
    fd.close()

    dst = os.path.join(str(tmpdir), 'dst')

    with py.test.raises(oz.OzException.OzException):
        guest._geteltorito(src, dst)

def test_geteltorito_bogus_boot_isoident(tmpdir):
    guest = setup_guest(tdlxml)

    src = os.path.join(str(tmpdir), 'src')
    fd = open(src, 'w')
    fd.seek(16*2048)
    fd.write("\x01")
    fd.write("CD001")
    fd.write("\x01")
    fd.write("\x00")
    fd.write("AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA")
    fd.write("BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBB")
    fd.write("\x00")
    fd.write('\0'*127)
    fd.seek(17*2048)
    fd.write("\x00")
    fd.write("AAAAA")
    fd.write('\0'*75)
    fd.close()

    dst = os.path.join(str(tmpdir), 'dst')

    with py.test.raises(oz.OzException.OzException):
        guest._geteltorito(src, dst)

def test_geteltorito_bogus_boot_version(tmpdir):
    guest = setup_guest(tdlxml)

    src = os.path.join(str(tmpdir), 'src')
    fd = open(src, 'w')
    fd.seek(16*2048)
    fd.write("\x01")
    fd.write("CD001")
    fd.write("\x01")
    fd.write("\x00")
    fd.write("AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA")
    fd.write("BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBB")
    fd.write("\x00")
    fd.write('\0'*127)
    fd.seek(17*2048)
    fd.write("\x00")
    fd.write("CD001")
    fd.write('\0'*75)
    fd.close()

    dst = os.path.join(str(tmpdir), 'dst')

    with py.test.raises(oz.OzException.OzException):
        guest._geteltorito(src, dst)

def test_geteltorito_bogus_boot_torito(tmpdir):
    guest = setup_guest(tdlxml)

    src = os.path.join(str(tmpdir), 'src')
    fd = open(src, 'w')
    fd.seek(16*2048)
    fd.write("\x01")
    fd.write("CD001")
    fd.write("\x01")
    fd.write("\x00")
    fd.write("AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA")
    fd.write("BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBB")
    fd.write("\x00")
    fd.write('\0'*127)
    fd.seek(17*2048)
    fd.write("\x00")
    fd.write("CD001")
    fd.write("\x01")
    fd.write('\0'*75)
    fd.close()

    dst = os.path.join(str(tmpdir), 'dst')

    with py.test.raises(oz.OzException.OzException):
        guest._geteltorito(src, dst)

def test_geteltorito_bogus_bootp(tmpdir):
    guest = setup_guest(tdlxml)

    src = os.path.join(str(tmpdir), 'src')
    fd = open(src, 'w')
    fd.seek(16*2048)
    fd.write("\x01")
    fd.write("CD001")
    fd.write("\x01")
    fd.write("\x00")
    fd.write("AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA")
    fd.write("BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBB")
    fd.write("\x00")
    fd.write('\0'*127)
    fd.seek(17*2048)
    fd.write("\x00")
    fd.write("CD001")
    fd.write("\x01")
    fd.write("EL TORITO SPECIFICATION")
    fd.write('\0'*41)
    fd.write("\x20\x00\x00\x00")
    fd.close()

    dst = os.path.join(str(tmpdir), 'dst')

    with py.test.raises(Exception):
        guest._geteltorito(src, dst)

def test_init_guest():
    guest = setup_guest(tdlxml2)

    assert guest.disksize == 20
    assert guest.image_name() == 'tester'
    assert guest.output_image_path() == '/home/%s/.oz/images/tester.dsk' % getpass.getuser()
    assert guest.default_auto_file() == True

def test_init_guest_bad_arch():
    tdl = oz.TDL.TDL(tdlxml)
    tdl.arch = 'armhf'  # Done here to make sure the TDL class doesn't error
    config = configparser.SafeConfigParser()
    config.readfp(StringIO("[libvirt]\nuri=qemu:///session\nbridge_name=%s" % route))
    with py.test.raises(Exception):
        oz.GuestFactory.guest_factory(tdl, config, None)

def test_icicle_generation():
    guest = setup_guest(tdlxml)
    with open(os.path.dirname(__file__) + '/test.icicle', 'r') as handle:
        test_icicle = handle.read()

    packages = [
        "accountsservice",
        "adduser",
        "apparmor",
        "apt",
        "apt-transport-https",
        "apt-utils",
        "apt-xapian-index",
        "aptitude",
        "at",
        "base-files",
        "base-passwd",
        "bash",
        "bash-completion",
        "bind9-host",
        "binutils",
        "bsdmainutils",
        "bsdutils",
        "build-essential",
        "busybox-initramfs",
        "busybox-static",
        "bzip2",
        "ca-certificates",
        "chef",
        "cloud-init",
        "cloud-initramfs-growroot",
        "cloud-initramfs-rescuevol",
        "cloud-utils",
        "comerr-dev",
        "console-setup",
        "coreutils",
        "cpio",
        "cpp",
        "cpp-4.6",
        "crda",
        "cron",
        "curl",
        "dash",
        "dbus",
        "debconf",
        "debconf-i18n",
        "debianutils",
        "denyhosts",
        "diffutils",
        "discover",
        "discover-data",
        "dkms",
        "dmidecode",
        "dmsetup",
        "dnsutils",
        "dosfstools",
        "dpkg",
        "dpkg-dev",
        "e2fslibs",
        "e2fsprogs",
        "ed",
        "eject",
        "euca2ools",
        "fakeroot",
        "file",
        "findutils",
        "friendly-recovery",
        "ftp",
        "fuse",
        "g++",
        "g++-4.6"
    ]

    icicle = guest._output_icicle_xml(packages, 'Icicle Description')
    assert test_icicle == icicle

def test_xml_generation_1():
    # Provide a macaddress so testing is easier
    guest = setup_guest(tdlxml, macaddress='52:54:00:04:cc:a6')

    with open(os.path.dirname(__file__) + '/libvirt/test_xml_generation_1.xml', 'r') as handle:
        test_xml = handle.read()

    # Replace various smaller items as they are auto generated
    test_xml = test_xml % (guest.uuid, route, guest.listen_port, guest.diskimage)

    bootdev = 'hd'
    installdev = None
    libvirt = guest._generate_xml(bootdev, installdev)

    assert test_xml == libvirt

def test_xml_generation_2():
    # Provide a macaddress so testing is easier
    guest = setup_guest(tdlxml, macaddress='52:54:00:04:cc:a6')

    with open(os.path.dirname(__file__) + '/libvirt/test_xml_generation_2.xml', 'r') as handle:
        test_xml = handle.read()

    # Replace various smaller items as they are auto generated
    test_xml = test_xml % (guest.uuid, route, guest.listen_port, guest.diskimage)

    bootdev = 'hd'
    installdev = guest._InstallDev('blue', '/var/bin/foo', 'muni')
    libvirt = guest._generate_xml(bootdev, installdev, kernel='kernel option', initrd='initrd option', cmdline='command line')

    assert test_xml == libvirt

def test_get_disks_and_interfaces():
    # Provide a macaddress so testing is easier
    guest = setup_guest(tdlxml, macaddress='52:54:00:04:cc:a6')

    path = os.path.dirname(__file__) + '/libvirt/test_get_disks_and_interfaces.xml'

    # Get the comparision xml
    with open(path, 'r') as handle:
        # Replace various smaller items as they are auto generated
        test_xml = handle.read() % (guest.uuid, route, guest.listen_port, guest.diskimage)
        disks, interfaces = guest._get_disks_and_interfaces(test_xml)

    assert disks == ['vda']
    assert interfaces == ['vnet7']

def test_get_disks_and_interfaces_missing_interface_target():
    # Provide a macaddress so testing is easier
    guest = setup_guest(tdlxml, macaddress='52:54:00:04:cc:a6')

    path = os.path.dirname(__file__) + '/libvirt/test_get_disks_and_interfaces_missing_interface_target.xml'

    # Get the comparision xml
    with open(path, 'r') as handle:
        with py.test.raises(Exception):
            # Replace various smaller items as they are auto generated
            test_xml = handle.read() % (guest.uuid, route, guest.listen_port, guest.diskimage)
            guest._get_disks_and_interfaces(test_xml)

def test_get_disks_and_interfaces_missing_interface_target_device():
    # Provide a macaddress so testing is easier
    guest = setup_guest(tdlxml, macaddress='52:54:00:04:cc:a6')

    path = os.path.dirname(__file__) + '/libvirt/test_get_disks_and_interfaces_missing_interface_target_device.xml'

    # Get the comparision xml
    with open(path, 'r') as handle:
        with py.test.raises(Exception):
            # Replace various smaller items as they are auto generated
            test_xml = handle.read() % (guest.uuid, route, guest.listen_port, guest.diskimage)
            guest._get_disks_and_interfaces(test_xml)

def test_get_disks_and_interfaces_missing_disk_target():
    # Provide a macaddress so testing is easier
    guest = setup_guest(tdlxml, macaddress='52:54:00:04:cc:a6')

    path = os.path.dirname(__file__) + '/libvirt/test_get_disks_and_interfaces_missing_disk_target.xml'

    # Get the comparision xml
    with open(path, 'r') as handle:
        with py.test.raises(Exception):
            # Replace various smaller items as they are auto generated
            test_xml = handle.read() % (guest.uuid, route, guest.listen_port, guest.diskimage)
            guest._get_disks_and_interfaces(test_xml)

def test_get_disks_and_interfaces_missing_disk_target_device():
    # Provide a macaddress so testing is easier
    guest = setup_guest(tdlxml, macaddress='52:54:00:04:cc:a6')

    # Get the comparision xml
    path = os.path.dirname(__file__) + '/libvirt/test_get_disks_and_interfaces_missing_disk_target_device.xml'
    with open(path, 'r') as handle:
        with py.test.raises(Exception):
            # Replace various smaller items as they are auto generated
            test_xml = handle.read() % (guest.uuid, route, guest.listen_port, guest.diskimage)
            guest._get_disks_and_interfaces(test_xml)

def test_modify_libvirt_xml_for_serial():
    # Provide a macaddress so testing is easier
    guest = setup_guest(tdlxml, macaddress='52:54:00:04:cc:a6')

    # Get the comparision xml
    path = os.path.dirname(__file__) + '/libvirt/test_modify_libvirt_xml_for_serial.xml'
    with open(path, 'r') as handle:
        # Replace various smaller items as they are auto generated
        test_xml = handle.read() % (guest.uuid, route, guest.listen_port, guest.diskimage)
        final = guest._modify_libvirt_xml_for_serial(test_xml)

    path = os.path.dirname(__file__) + '/libvirt/test_modify_libvirt_xml_for_serial_final.xml'
    with open(path, 'r') as handle:
        # Replace various smaller items as they are auto generated
        final_xml = handle.read() % (guest.uuid, route, guest.diskimage, guest.listen_port)
        assert final_xml == final

def test_modify_libvirt_xml_for_serial_too_many_targets():
    # Provide a macaddress so testing is easier
    guest = setup_guest(tdlxml, macaddress='52:54:00:04:cc:a6')

    # Get the comparision xml
    path = os.path.dirname(__file__) + '/libvirt/test_modify_libvirt_xml_for_serial_too_many_targets.xml'
    with open(path, 'r') as handle:
        with py.test.raises(Exception):
            # Replace various smaller items as they are auto generated
            test_xml = handle.read() % (guest.uuid, route, guest.listen_port, guest.diskimage)
            guest._modify_libvirt_xml_for_serial(test_xml)

def test_modify_libvirt_xml_for_serial_missing_devices():
    # Provide a macaddress so testing is easier
    guest = setup_guest(tdlxml, macaddress='52:54:00:04:cc:a6')

    # Get the comparision xml
    path = os.path.dirname(__file__) + '/libvirt/test_modify_libvirt_xml_for_serial_missing_devices.xml'
    with open(path, 'r') as handle:
        with py.test.raises(Exception):
            # Replace various smaller items as they are auto generated
            test_xml = handle.read() % (guest.uuid, route, guest.listen_port, guest.diskimage)
            guest._modify_libvirt_xml_for_serial(test_xml)

def test_modify_libvirt_xml_for_serial_too_many_devices():
    # Provide a macaddress so testing is easier
    guest = setup_guest(tdlxml, macaddress='52:54:00:04:cc:a6')

    # Get the comparision xml
    path = os.path.dirname(__file__) + '/libvirt/test_modify_libvirt_xml_for_serial_too_many_devices.xml'
    with open(path, 'r') as handle:
        with py.test.raises(Exception):
            # Replace various smaller items as they are auto generated
            test_xml = handle.read() % (guest.uuid, route, guest.listen_port, guest.diskimage)
            guest._modify_libvirt_xml_for_serial(test_xml)


def test_modify_libvirt_xml_diskimage():
    # Provide a macaddress so testing is easier
    guest = setup_guest(tdlxml, macaddress='52:54:00:04:cc:a6')

    # Get the comparision xml
    path = os.path.dirname(__file__) + '/libvirt/test_modify_libvirt_xml_diskimage.xml'
    with open(path, 'r') as handle:
        # Replace various smaller items as they are auto generated
        test_xml = handle.read() % (guest.uuid, route, guest.listen_port, guest.diskimage)

        name, ext = os.path.splitext(guest.diskimage)
        image = name + '.qcow2'
        final = guest._modify_libvirt_xml_diskimage(test_xml, image, 'qcow2')

    path = os.path.dirname(__file__) + '/libvirt/test_modify_libvirt_xml_diskimage_final.xml'
    with open(path, 'r') as handle:
        # Replace various smaller items as they are auto generated
        final_xml = handle.read() % (guest.uuid, route, guest.listen_port, image)
        assert final_xml == final

def test_modify_libvirt_xml_diskimage_missing_disk_source():
    # Provide a macaddress so testing is easier
    guest = setup_guest(tdlxml, macaddress='52:54:00:04:cc:a6')

    # Get the comparision xml
    path = os.path.dirname(__file__) + '/libvirt/test_modify_libvirt_xml_diskimage_missing_disk_source.xml'
    with open(path, 'r') as handle:
        with py.test.raises(Exception):
            # Replace various smaller items as they are auto generated
            test_xml = handle.read() % (guest.uuid, route, guest.listen_port, guest.diskimage)
            guest._modify_libvirt_xml_diskimage(test_xml, guest.diskimage, 'qcow2')

def test_modify_libvirt_xml_diskimage_too_many_drivers():
    # Provide a macaddress so testing is easier
    guest = setup_guest(tdlxml, macaddress='52:54:00:04:cc:a6')

    # Get the comparision xml
    path = os.path.dirname(__file__) + '/libvirt/test_modify_libvirt_xml_diskimage_too_many_drivers.xml'
    with open(path, 'r') as handle:
        with py.test.raises(Exception):
            # Replace various smaller items as they are auto generated
            test_xml = handle.read() % (guest.uuid, route, guest.listen_port, guest.diskimage)
            guest._modify_libvirt_xml_diskimage(test_xml, guest.diskimage, 'qcow2')



# Copyright (C) 2010,2011  Chris Lalancette <clalance@redhat.com>
# Copyright (C) 2012-2018  Chris Lalancette <clalancette@gmail.com>

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation;
# version 2.1 of the License.

# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

"""
Fedora Core installation
"""

import os

import oz.OzException
import oz.RedHat
import oz.ozutil


class FedoraCoreConfiguration(object):
    """
    Configuration class for Fedora Core installation.
    """
    def __init__(self, initrdtype):
        self._initrdtype = initrdtype

    @property
    def initrdtype(self):
        """
        Property method for the type of initrd this version of Fedora uses
        ('cpio' or 'ext2').
        """
        return self._initrdtype


version_to_config = {
    '6': FedoraCoreConfiguration(initrdtype='cpio'),
    '5': FedoraCoreConfiguration(initrdtype='cpio'),
    '4': FedoraCoreConfiguration(initrdtype='cpio'),
    '3': FedoraCoreConfiguration(initrdtype='ext2'),
    '2': FedoraCoreConfiguration(initrdtype='ext2'),
    '1': FedoraCoreConfiguration(initrdtype='ext2'),
}


class FedoraCoreGuest(oz.RedHat.RedHatLinuxCDGuest):
    """
    Class for Fedora Core 1, 2, 3, 4, 5, and 6 installation.
    """
    def __init__(self, tdl, config, auto, output_disk, netdev, diskbus,
                 macaddress):
        self.config = version_to_config[tdl.update]
        oz.RedHat.RedHatLinuxCDGuest.__init__(self, tdl, config, auto,
                                              output_disk, netdev, diskbus,
                                              True, True, self.config.initrdtype,
                                              macaddress)

        # FIXME: if doing an ISO install, we have to check that the ISO passed
        # in is the DVD, not the CD (since we can't change disks midway)

    def _modify_iso(self):
        """
        Method to modify the ISO for autoinstallation.
        """
        self._copy_kickstart(os.path.join(self.iso_contents, "ks.cfg"))

        initrdline = "  append initrd=initrd.img ks=cdrom:/ks.cfg method="
        if self.tdl.installtype == "url":
            initrdline += self.url
        else:
            initrdline += "cdrom:/dev/cdrom"
        self._modify_isolinux(initrdline)

    def get_auto_path(self):
        """
        Method to create the correct path to the Fedora Core kickstart files.
        """
        return oz.ozutil.generate_full_auto_path("FedoraCore" + self.tdl.update + ".auto")


def get_class(tdl, config, auto, output_disk=None, netdev=None, diskbus=None,
              macaddress=None):
    """
    Factory method for Fedora Core installs.
    """
    if tdl.update in version_to_config.keys():
        return FedoraCoreGuest(tdl, config, auto, output_disk, netdev, diskbus,
                               macaddress)


def get_supported_string():
    """
    Return supported versions as a string.
    """
    return "Fedora Core: " + ", ".join(sorted(version_to_config.keys()))



# Copyright (C) 2011  Chris Lalancette <clalance@redhat.com>
# Copyright (C) 2012-2017  Chris Lalancette <clalancette@gmail.com>

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation;
# version 2.1 of the License.

# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

"""
Factory functions.
"""

import oz.OzException

os_dict = {
    'Fedora': 'Fedora',
    'FedoraCore': 'FedoraCore',
    'FC': 'FedoraCore',
    'RedHatEnterpriseLinux-2.1': 'RHEL_2_1',
    'RHEL-2.1': 'RHEL_2_1',
    'RedHatEnterpriseLinux-3': 'RHEL_3',
    'RHEL-3': 'RHEL_3',
    'CentOS-3': 'RHEL_3',
    'RedHatEnterpriseLinux-4': 'RHEL_4',
    'RHEL-4': 'RHEL_4',
    'CentOS-4': 'RHEL_4',
    'ScientificLinux-4': 'RHEL_4',
    'SL-4': 'RHEL_4',
    'RedHatEnterpriseLinux-5': 'RHEL_5',
    'RHEL-5': 'RHEL_5',
    'CentOS-5': 'RHEL_5',
    'OL-5': 'RHEL_5',
    'ScientificLinux-5': 'RHEL_5',
    'SL-5': 'RHEL_5',
    'ScientificLinuxCern-5': 'RHEL_5',
    'SLC-5': 'RHEL_5',
    'RedHatEnterpriseLinux-6': 'RHEL_6',
    'RHEL-6': 'RHEL_6',
    'CentOS-6': 'RHEL_6',
    'ScientificLinux-6': 'RHEL_6',
    'SL-6': 'RHEL_6',
    'ScientificLinuxCern-6': 'RHEL_6',
    'SLC-6': 'RHEL_6',
    'OracleEnterpriseLinux-6': 'RHEL_6',
    'OEL-6': 'RHEL_6',
    'OL-6': 'RHEL_6',
    'RHEL-7': 'RHEL_7',
    'CentOS-7': 'RHEL_7',
    'Ubuntu': 'Ubuntu',
    'Windows': 'Windows',
    'RedHatLinux': 'RHL',
    'RHL': 'RHL',
    'OpenSUSE': 'OpenSUSE',
    'Debian': 'Debian',
    'Mandrake': 'Mandrake',
    'Mandriva': 'Mandriva',
    'Mageia': 'Mageia',
    'FreeBSD': 'FreeBSD',
}


def guest_factory(tdl, config, auto, output_disk=None, netdev=None,
                  diskbus=None, macaddress=None):
    """
    Factory function return an appropriate Guest object based on the TDL.
    The arguments are:

    tdl    - The TDL object to be used.  The return object will be determined
             based on the distro and version from the TDL.
    config - A ConfigParser object that contains configuration.  If None is
             passed for the config, Oz defaults will be used.
    auto   - An unattended installation file to be used for the
             installation.  If None is passed for auto, then Oz will use
             a known-working unattended installation file.
    output_disk - An optional string argument specifying the path to the
                  disk to be written to.
    netdev - An optional string argument specifying the type of network device
             to be used during installation.  If specified, this will override
             the default that Oz uses.
    diskbus - An optional string argument specifying the type of disk device
              to be used during installation.  If specified, this will override
              the default that Oz uses.
    macaddress - An optional string argument specifying the MAC address to use
                 for the guest.
    """

    klass = None
    for name, importname in os_dict.items():
        if tdl.distro == name:
            # we found the matching module; import and call the get_class method
            module = __import__('oz.' + importname)
            klass = getattr(module, importname).get_class(tdl, config, auto,
                                                          output_disk, netdev,
                                                          diskbus, macaddress)
            break

    if klass is None:
        raise oz.OzException.OzException("Unsupported " + tdl.distro + " update " + tdl.update)

    return klass


def distrolist():
    """
    Function to print out a list of supported distributions.
    """
    strings = []
    for importname in os_dict.values():
        module = __import__('oz.' + importname)
        support = getattr(module, importname).get_supported_string()
        tmp = '   ' + support
        if tmp not in strings:
            strings.append(tmp)

    strings.sort()
    print('\n'.join(strings))



#!/usr/bin/python

import sys
import os

try:
    import py.test
except ImportError:
    print('Unable to import py.test.  Is py.test installed?')
    sys.exit(1)

# Find oz
prefix = '.'
for i in range(0,3):
    if os.path.isdir(os.path.join(prefix, 'oz')):
        sys.path.insert(0, prefix)
        break
    else:
        prefix = '../' + prefix

try:
    import oz.ozutil
except ImportError:
    print('Unable to import oz.  Is oz installed?')
    sys.exit(1)

# test oz.ozutil.generate_full_auto_path
def test_auto():
    assert(oz.ozutil.generate_full_auto_path('fedora-14-jeos.ks') == os.path.realpath(os.path.join(prefix, 'oz', 'auto', 'fedora-14-jeos.ks')))

def test_auto_none():
    with py.test.raises(Exception):
        oz.ozutil.generate_full_auto_path(None)

# test oz.ozutil.executable_exists
def test_exe_exists_bin_ls():
    assert(oz.ozutil.executable_exists('/bin/ls'))

def test_exe_exists_foo():
    with py.test.raises(Exception):
        oz.ozutil.executable_exists('foo')

def test_exe_exists_full_foo():
    with py.test.raises(Exception):
        oz.ozutil.executable_exists('/bin/foo')

def test_exe_exists_not_x():
    with py.test.raises(Exception):
        oz.ozutil.executable_exists('/etc/hosts')

def test_exe_exists_relative_false():
    assert(oz.ozutil.executable_exists('false'))

def test_exe_exists_none():
    with py.test.raises(Exception):
        oz.ozutil.executable_exists(None)

# test oz.ozutil.copyfile_sparse
def test_copy_sparse_none_src():
    with py.test.raises(Exception):
        oz.ozutil.copyfile_sparse(None, None)

def test_copy_sparse_none_dst(tmpdir):
    fullname = os.path.join(str(tmpdir), 'src')
    open(fullname, 'w').write('src')
    with py.test.raises(Exception):
        oz.ozutil.copyfile_sparse(fullname, None)

def test_copy_sparse_bad_src_mode(tmpdir):
    if os.geteuid() == 0:
        # this test won't work as root, since root can copy any mode files
        return
    fullname = os.path.join(str(tmpdir), 'writeonly')
    open(fullname, 'w').write('writeonly')
    os.chmod(fullname, 0000)
    # because copyfile_sparse uses os.open() instead of open(), it throws an
    # OSError
    with py.test.raises(OSError):
        oz.ozutil.copyfile_sparse(fullname, 'output')

def test_copy_sparse_bad_dst_mode(tmpdir):
    if os.geteuid() == 0:
        # this test won't work as root, since root can copy any mode files
        return
    srcname = os.path.join(str(tmpdir), 'src')
    open(srcname, 'w').write('src')
    dstname = os.path.join(str(tmpdir), 'dst')
    open(dstname, 'w').write('dst')
    os.chmod(dstname, 0o444)
    with py.test.raises(OSError):
        oz.ozutil.copyfile_sparse(srcname, dstname)

def test_copy_sparse_zero_size_src(tmpdir):
    srcname = os.path.join(str(tmpdir), 'src')
    fd = open(srcname, 'w')
    fd.close()
    dstname = os.path.join(str(tmpdir), 'dst')
    oz.ozutil.copyfile_sparse(srcname, dstname)
    assert(os.stat(dstname).st_size == 0)

def test_copy_sparse_small_src(tmpdir):
    srcname = os.path.join(str(tmpdir), 'src')
    open(srcname, 'w').write('src')
    dstname = os.path.join(str(tmpdir), 'dst')
    oz.ozutil.copyfile_sparse(srcname, dstname)
    assert(os.stat(dstname).st_size == 3)

def test_copy_sparse_one_block_src(tmpdir):
    with open('/dev/urandom', 'rb') as infd:
        # we read 32*1024 to make sure we use one big buf_size block (see the
        # implementation of copyfile_sparse)
        data = infd.read(32*1024)

    srcname = os.path.join(str(tmpdir), 'src')
    with open(srcname, 'wb') as outfd:
        outfd.write(data)
    dstname = os.path.join(str(tmpdir), 'dst')
    oz.ozutil.copyfile_sparse(srcname, dstname)
    assert(os.stat(dstname).st_size == 32*1024)

def test_copy_sparse_many_blocks_src(tmpdir):
    with open('/dev/urandom', 'rb') as infd:
        # we read 32*1024 to make sure we use one big buf_size block (see the
        # implementation of copyfile_sparse)
        data = infd.read(32*1024*10)

    srcname = os.path.join(str(tmpdir), 'src')
    with open(srcname, 'wb') as outfd:
        outfd.write(data)
    dstname = os.path.join(str(tmpdir), 'dst')
    oz.ozutil.copyfile_sparse(srcname, dstname)
    assert(os.stat(dstname).st_size == 32*1024*10)

def test_copy_sparse_zero_blocks(tmpdir):
    with open('/dev/urandom', 'rb') as infd:
        # we read 32*1024 to make sure we use one big buf_size block (see the
        # implementation of copyfile_sparse)
        data1 = infd.read(32*1024)
        data2 = infd.read(32*1024)

    srcname = os.path.join(str(tmpdir), 'src')
    with open(srcname, 'wb') as outfd:
        outfd.write(data1)
        outfd.write(b'\0'*32*1024)
        outfd.write(data2)
    dstname = os.path.join(str(tmpdir), 'dst')
    oz.ozutil.copyfile_sparse(srcname, dstname)
    assert(os.stat(dstname).st_size == 32*1024*3)

def test_copy_sparse_src_not_exists(tmpdir):
    srcname = os.path.join(str(tmpdir), 'src')
    dstname = os.path.join(str(tmpdir), 'dst')
    open(dstname, 'w').write('dst')
    with py.test.raises(Exception):
        oz.ozutil.copyfile_sparse(srcname, dstname)

def test_copy_sparse_dest_not_exists(tmpdir):
    srcname = os.path.join(str(tmpdir), 'src')
    open(srcname, 'w').write('src')
    dstname = os.path.join(str(tmpdir), 'dst')
    oz.ozutil.copyfile_sparse(srcname, dstname)
    assert(os.stat(dstname).st_size == 3)

def test_copy_sparse_src_is_dir(tmpdir):
    dstname = os.path.join(str(tmpdir), 'dst')
    open(dstname, 'w').write('dst')
    with py.test.raises(Exception):
        oz.ozutil.copyfile_sparse(tmpdir, dstname)

def test_copy_sparse_dst_is_dir(tmpdir):
    srcname = os.path.join(str(tmpdir), 'src')
    open(srcname, 'w').write('src')
    with py.test.raises(Exception):
        oz.ozutil.copyfile_sparse(srcname, tmpdir)

# test oz.ozutil.string_to_bool
def test_stb_no():
    for nletter in ['n', 'N']:
        for oletter in ['o', 'O']:
            curr = nletter+oletter
            assert(oz.ozutil.string_to_bool(curr) == False)

def test_stb_false():
    for fletter in ['f', 'F']:
        for aletter in ['a', 'A']:
            for lletter in ['l', 'L']:
                for sletter in ['s', 'S']:
                    for eletter in ['e', 'E']:
                        curr = fletter+aletter+lletter+sletter+eletter
                        assert(oz.ozutil.string_to_bool(curr) == False)

def test_stb_yes():
    for yletter in ['y', 'Y']:
        for eletter in ['e', 'E']:
            for sletter in ['s', 'S']:
                curr = yletter+eletter+sletter
                assert(oz.ozutil.string_to_bool(curr) == True)

def test_stb_true():
    for tletter in ['t', 'T']:
        for rletter in ['r', 'R']:
            for uletter in ['u', 'U']:
                for eletter in ['e', 'E']:
                    curr = tletter+rletter+uletter+eletter
                    assert(oz.ozutil.string_to_bool(curr) == True)

def test_stb_none():
    with py.test.raises(Exception):
        oz.ozutil.string_to_bool(None)


def test_stb_invalid():
    assert(oz.ozutil.string_to_bool('foobar') is None)

# test oz.ozutil.generate_macaddress
def test_genmac():
    mac = oz.ozutil.generate_macaddress()
    assert(type(mac) == str)
    assert(len(mac) == 17)

# test oz.ozutil.mkdir_p
def test_mkdir_p(tmpdir):
    fullname = os.path.join(str(tmpdir), 'foo')
    oz.ozutil.mkdir_p(fullname)

def test_mkdir_p_twice(tmpdir):
    fullname = os.path.join(str(tmpdir), 'foo')
    oz.ozutil.mkdir_p(fullname)
    oz.ozutil.mkdir_p(fullname)

def test_mkdir_p_file_exists(tmpdir):
    fullname = os.path.join(str(tmpdir), 'file_exists')
    open(fullname, 'w').write('file_exists')
    with py.test.raises(OSError):
        oz.ozutil.mkdir_p(fullname)

def test_mkdir_p_none():
    with py.test.raises(Exception):
        oz.ozutil.mkdir_p(None)

def test_mkdir_p_empty_string():
    oz.ozutil.mkdir_p('')

# test oz.ozutil.copy_modify_file
def test_copy_modify_none_src():
    with py.test.raises(Exception):
        oz.ozutil.copy_modify_file(None, None, None)

def test_copy_modify_none_dst(tmpdir):
    fullname = os.path.join(str(tmpdir), 'src')
    open(fullname, 'w').write('src')
    with py.test.raises(Exception):
        oz.ozutil.copy_modify_file(fullname, None, None)

def test_copy_modify_none_subfunc(tmpdir):
    src = os.path.join(str(tmpdir), 'src')
    open(src, 'w').write('src')
    dst = os.path.join(str(tmpdir), 'dst')
    with py.test.raises(Exception):
        oz.ozutil.copy_modify_file(src, dst, None)

def test_copy_modify_bad_src_mode(tmpdir):
    if os.geteuid() == 0:
        # this test won't work as root, since root can copy any mode files
        return
    def sub(line):
        return line
    fullname = os.path.join(str(tmpdir), 'writeonly')
    open(fullname, 'w').write('writeonly')
    os.chmod(fullname, 0000)
    dst = os.path.join(str(tmpdir), 'dst')
    with py.test.raises(IOError):
        oz.ozutil.copy_modify_file(fullname, dst, sub)

def test_copy_modify_empty_file(tmpdir):
    def sub(line):
        return line
    src = os.path.join(str(tmpdir), 'src')
    f = open(src, 'w')
    f.close()
    dst = os.path.join(str(tmpdir), 'dst')
    oz.ozutil.copy_modify_file(src, dst, sub)
    assert(os.stat(dst).st_size == 0)

def test_copy_modify_file(tmpdir):
    def sub(line):
        return line
    src = os.path.join(str(tmpdir), 'src')
    with open(src, 'w') as f:
        f.write("this is a line in the file\n")
        f.write("this is another line in the file\n")
    dst = os.path.join(str(tmpdir), 'dst')
    oz.ozutil.copy_modify_file(src, dst, sub)
    assert(os.stat(dst).st_size == 60)

# test oz.ozutil.write_cpio
def test_write_cpio_none_input():
    with py.test.raises(Exception):
        oz.ozutil.write_cpio(None, None)

def test_write_cpio_none_output():
    with py.test.raises(Exception):
        oz.ozutil.write_cpio({}, None)

def test_write_cpio_empty_dict(tmpdir):
    dst = os.path.join(str(tmpdir), 'dst')
    oz.ozutil.write_cpio({}, dst)

def test_write_cpio_existing_file(tmpdir):
    if os.geteuid() == 0:
        # this test won't work as root, since root can copy any mode files
        return
    dst = os.path.join(str(tmpdir), 'dst')
    open(dst, 'w').write('hello')
    os.chmod(dst, 0000)
    with py.test.raises(IOError):
        oz.ozutil.write_cpio({}, dst)

def test_write_cpio_single_file(tmpdir):
    src = os.path.join(str(tmpdir), 'src')
    open(src, 'w').write('src')
    dst = os.path.join(str(tmpdir), 'dst')
    oz.ozutil.write_cpio({src: 'src'}, dst)

def test_write_cpio_multiple_files(tmpdir):
    src1 = os.path.join(str(tmpdir), 'src1')
    open(src1, 'w').write('src1')
    src2 = os.path.join(str(tmpdir), 'src2')
    open(src2, 'w').write('src2')
    dst = os.path.join(str(tmpdir), 'dst')
    oz.ozutil.write_cpio({src1: 'src1', src2: 'src2'}, dst)

def test_write_cpio_not_multiple_of_4(tmpdir):
    src = os.path.join(str(tmpdir), 'src')
    open(src, 'w').write('src')
    dst = os.path.join(str(tmpdir), 'dst')
    oz.ozutil.write_cpio({src: 'src'}, dst)

def test_write_cpio_exception(tmpdir):
    if os.geteuid() == 0:
        # this test won't work as root, since root can copy any mode files
        return
    src = os.path.join(str(tmpdir), 'src')
    open(src, 'w').write('src')
    os.chmod(src, 0000)
    dst = os.path.join(str(tmpdir), 'dst')
    with py.test.raises(IOError):
        oz.ozutil.write_cpio({src: 'src'}, dst)

def test_md5sum_regular(tmpdir):
    src = os.path.join(str(tmpdir), 'md5sum')
    with open(src, 'w') as f:
        f.write('# this is a comment line, followed by a blank line\n\n6e812e782e52b536c0307bb26b3c244e *Fedora-11-i386-DVD.iso\n')

    oz.ozutil.get_md5sum_from_file(src, 'Fedora-11-i386-DVD.iso')

def test_sha1sum_regular(tmpdir):
    src = os.path.join(str(tmpdir), 'md5sum')
    with open(src, 'w') as f:
        f.write('6e812e782e52b536c0307bb26b3c244e1c42b644 *Fedora-11-i386-DVD.iso\n')

    oz.ozutil.get_sha1sum_from_file(src, 'Fedora-11-i386-DVD.iso')

def test_sha256sum_regular(tmpdir):
    src = os.path.join(str(tmpdir), 'md5sum')
    with open(src, 'w') as f:
        f.write('6e812e782e52b536c0307bb26b3c244e1c42b644235f5a4b242786b1ef375358 *Fedora-11-i386-DVD.iso\n')

    oz.ozutil.get_sha256sum_from_file(src, 'Fedora-11-i386-DVD.iso')

def test_md5sum_bsd(tmpdir):
    src = os.path.join(str(tmpdir), 'md5sum')
    with open(src, 'w') as f:
        f.write('MD5 (Fedora-11-i386-DVD.iso)=6e812e782e52b536c0307bb26b3c244e1c42b644235f5a4b242786b1ef375358\n')

    oz.ozutil.get_md5sum_from_file(src, 'Fedora-11-i386-DVD.iso')

def test_md5sum_bsd_no_start_paren(tmpdir):
    src = os.path.join(str(tmpdir), 'md5sum')
    with open(src, 'w') as f:
        # if BSD is missing a paren, we don't raise an exception, just ignore and
        # continue
        f.write('MD5 Fedora-11-i386-DVD.iso)=6e812e782e52b536c0307bb26b3c244e1c42b644235f5a4b242786b1ef375358\n')

    oz.ozutil.get_md5sum_from_file(src, 'Fedora-11-i386-DVD.iso')

def test_md5sum_bsd_no_end_paren(tmpdir):
    src = os.path.join(str(tmpdir), 'md5sum')
    with open(src, 'w') as f:
        # if BSD is missing a paren, we don't raise an exception, just ignore and
        # continue
        f.write('MD5 (Fedora-11-i386-DVD.iso=6e812e782e52b536c0307bb26b3c244e1c42b644235f5a4b242786b1ef375358\n')

    oz.ozutil.get_md5sum_from_file(src, 'Fedora-11-i386-DVD.iso')

def test_md5sum_bsd_no_equal(tmpdir):
    src = os.path.join(str(tmpdir), 'md5sum')
    with open(src, 'w') as f:
        # if BSD is missing a paren, we don't raise an exception, just ignore and
        # continue
        f.write('MD5 (Fedora-11-i386-DVD.iso) 6e812e782e52b536c0307bb26b3c244e1c42b644235f5a4b242786b1ef375358\n')

    oz.ozutil.get_md5sum_from_file(src, 'Fedora-11-i386-DVD.iso')

def test_md5sum_regular_escaped(tmpdir):
    src = os.path.join(str(tmpdir), 'md5sum')
    with open(src, 'w') as f:
        f.write('\\6e812e782e52b536c0307bb26b3c244e *Fedora-11-i386-DVD.iso\n')

    oz.ozutil.get_md5sum_from_file(src, 'Fedora-11-i386-DVD.iso')

def test_md5sum_regular_too_short(tmpdir):
    src = os.path.join(str(tmpdir), 'md5sum')
    with open(src, 'w') as f:
        f.write('6e *F\n')

    oz.ozutil.get_md5sum_from_file(src, 'Fedora-11-i386-DVD.iso')

def test_md5sum_regular_no_star(tmpdir):
    src = os.path.join(str(tmpdir), 'md5sum')
    with open(src, 'w') as f:
        f.write('6e812e782e52b536c0307bb26b3c244e Fedora-11-i386-DVD.iso\n')

    oz.ozutil.get_md5sum_from_file(src, 'Fedora-11-i386-DVD.iso')

def test_md5sum_regular_no_newline(tmpdir):
    src = os.path.join(str(tmpdir), 'md5sum')
    with open(src, 'w') as f:
        f.write('6e812e782e52b536c0307bb26b3c244e *Fedora-11-i386-DVD.iso')

    oz.ozutil.get_md5sum_from_file(src, 'Fedora-11-i386-DVD.iso')

def test_md5sum_regular_no_space(tmpdir):
    src = os.path.join(str(tmpdir), 'md5sum')
    with open(src, 'w') as f:
        f.write('6e812e782e52b536c0307bb26b3c244e_*Fedora-11-i386-DVD.iso\n')

    oz.ozutil.get_md5sum_from_file(src, 'Fedora-11-i386-DVD.iso')



#!/usr/bin/python

from __future__ import print_function
import sys
try:
    import configparser
except ImportError:
    import ConfigParser as configparser
try:
    from StringIO import StringIO
except:
    from io import StringIO
import logging
import os

# Find oz library
prefix = '.'
for i in range(0,3):
    if os.path.isdir(os.path.join(prefix, 'oz')):
        sys.path.insert(0, prefix)
        break
    else:
        prefix = '../' + prefix

try:
    import oz.TDL
    import oz.GuestFactory
except ImportError as e:
    print(e)
    print('Unable to import oz.  Is oz installed or in your PYTHONPATH?')
    sys.exit(1)

try:
    import py.test
except ImportError:
    print('Unable to import py.test.  Is py.test installed?')
    sys.exit(1)

def default_route():
    route_file = "/proc/net/route"
    d = open(route_file)

    for line in d:
        info = line.split()
        if (len(info) != 11): # 11 = typical num of fields in the file
            logging.warn(_("Invalid line length while parsing %s.") %
                         (route_file))
            break
        try:
            route = int(info[1], 16)
            if route == 0:
                return info[0]
        except ValueError:
            continue
    raise Exception("Could not find default route")

# we find the default route for this machine.  Note that this very well
# may not be a bridge, but for the purposes of testing the factory, it
# doesn't really matter; it just has to have an IP address
route = default_route()

def runtest(**kwargs):
    distro = kwargs['distro']
    version = kwargs['version']
    arch = kwargs['arch']
    installtype = kwargs['installtype']
    expect_success = kwargs['expect_success']
    print("Testing %s-%s-%s-%s" % (distro, version, arch, installtype))

    tdlxml = """
<template>
  <name>tester</name>
  <os>
    <name>%s</name>
    <version>%s</version>
    <arch>%s</arch>
    <install type='%s'>
      <%s>file://example.org</%s>
    </install>
    <key>1234</key>
  </os>
</template>
""" % (distro, version, arch, installtype, installtype, installtype)

    success = None
    saved_exc = None
    try:
        tdl = oz.TDL.TDL(tdlxml)

        config = configparser.SafeConfigParser()
        config.readfp(StringIO("[libvirt]\nuri=qemu:///session\nbridge_name=%s" % route))

        if os.getenv('DEBUG') is not None:
            logging.basicConfig(level=logging.DEBUG, format="%(message)s")
        else:
            logging.basicConfig(level=logging.ERROR, format="%(message)s")

            oz.GuestFactory.guest_factory(tdl, config, None)
        if not expect_success:
            assert(False)
    except oz.OzException.OzException as e:
        if expect_success:
            raise

def test_bad_distro():
    runtest(distro='foo', version='1', arch='i386', installtype='url',
            expect_success=False)

def test_bad_installtype():
    runtest(distro='Fedora', version='14', arch='i386', installtype='duh',
            expect_success=False)

def test_bad_arch():
    runtest(distro='Fedora', version='14', arch='ia64', installtype='iso',
            expect_success=False)

def test_fedora_core():
    for version in ["1", "2", "3", "4", "5", "6"]:
        for arch in ["i386", "x86_64"]:
            for installtype in ["url", "iso"]:
                runtest(distro='FedoraCore', version=version, arch=arch,
                        installtype=installtype, expect_success=True)
    runtest(distro='FedoraCore', version='24', arch='x86_64', installtype='iso',
            expect_success=False)
    runtest(distro='FedoraCore', version='6', arch='ia64', installtype='iso',
            expect_success=False)
    runtest(distro='FedoraCore', version='6', arch='x86_64', installtype='foo',
            expect_success=False)

def test_fedora():
    for version in ["7", "8", "9", "10", "11", "12", "13", "14", "15", "16",
                    "17", "18", "19", "20", "21", "22", "23", "24", "25", "26",
                    "27", "28"]:
        for arch in ["i386", "x86_64"]:
            for installtype in ["url", "iso"]:
                runtest(distro='Fedora', version=version, arch=arch,
                        installtype=installtype, expect_success=True)
    runtest(distro='Fedora', version='1', arch='x86_64', installtype='iso',
            expect_success=False)
    runtest(distro='Fedora', version='1', arch='ia64', installtype='iso',
            expect_success=False)
    runtest(distro='Fedora', version='1', arch='x86_64', installtype='foo',
            expect_success=False)

def test_rhl():
    for version in ["7.0", "7.1", "7.2", "7.3", "8", "9"]:
        runtest(distro='RHL', version=version, arch='i386', installtype='url',
                expect_success=True)
    runtest(distro='RHL', version='10', arch='i386', installtype='url',
            expect_success=False)
    runtest(distro='RHL', version='9', arch='x86_64', installtype='url',
            expect_success=False)
    runtest(distro='RHL', version='9', arch='i386', installtype='iso',
            expect_success=False)

def test_rhel21():
    for version in ["GOLD", "U2", "U3", "U4", "U5", "U6"]:
        runtest(distro='RHEL-2.1', version=version, arch='i386',
                installtype='url', expect_success=True)
    runtest(distro='RHEL-2.1', version='U7', arch='i386', installtype='url',
            expect_success=False)
    runtest(distro='RHEL-2.1', version='U6', arch='x86_64', installtype='url',
            expect_success=False)
    runtest(distro='RHEL-2.1', version='U6', arch='i386', installtype='iso',
            expect_success=False)

def test_rhel3():
    for distro in ["RHEL-3", "CentOS-3"]:
        for version in ["GOLD", "U1", "U2", "U3", "U4", "U5", "U6", "U7", "U8",
                        "U9"]:
            for arch in ["i386", "x86_64"]:
                runtest(distro=distro, version=version, arch=arch,
                        installtype='url', expect_success=True)
    runtest(distro='RHEL-3', version='U10', arch='x86_64', installtype='url',
            expect_success=False)
    runtest(distro='RHEL-3', version='U9', arch='ia64', installtype='iso',
            expect_success=False)
    runtest(distro='RHEL-3', version='U9', arch='x86_64', installtype='foo',
            expect_success=False)

def test_rhel4():
    for distro in ["RHEL-4", "CentOS-4", "ScientificLinux-4"]:
        for version in ["GOLD", "U1", "U2", "U3", "U4", "U5", "U6", "U7", "U8",
                        "U9"]:
            for arch in ["i386", "x86_64"]:
                for installtype in ["url", "iso"]:
                    runtest(distro=distro, version=version, arch=arch,
                            installtype=installtype, expect_success=True)
    runtest(distro='RHEL-4', version='U10', arch='x86_64', installtype='url',
            expect_success=False)
    runtest(distro='RHEL-4', version='U9', arch='ia64', installtype='iso',
            expect_success=False)
    runtest(distro='RHEL-4', version='U9', arch='x86_64', installtype='foo',
            expect_success=False)

def test_rhel5():
    for distro in ["RHEL-5", "CentOS-5", "ScientificLinux-5"]:
        for version in ["GOLD", "U1", "U2", "U3", "U4", "U5", "U6", "U7", "U8",
                        "U9", "U10", "U11"]:
            for arch in ["i386", "x86_64"]:
                for installtype in ["url", "iso"]:
                    runtest(distro=distro, version=version, arch=arch,
                            installtype=installtype, expect_success=True)
    runtest(distro='RHEL-5', version='U20', arch='x86_64', installtype='url',
            expect_success=False)
    runtest(distro='RHEL-5', version='U9', arch='ia64', installtype='iso',
            expect_success=False)
    runtest(distro='RHEL-5', version='U9', arch='x86_64', installtype='foo',
            expect_success=False)

def test_rhel6():
    for distro in ["RHEL-6", "CentOS-6", "ScientificLinux-6", "OEL-6"]:
        for version in ["0", "1", "2", "3", "4", "5", "6", "7", "8"]:
            for arch in ["i386", "x86_64"]:
                for installtype in ["url", "iso"]:
                    runtest(distro=distro, version=version, arch=arch,
                            installtype=installtype, expect_success=True)
    runtest(distro='RHEL-6', version='U9', arch='x86_64', installtype='url',
            expect_success=False)
    runtest(distro='RHEL-6', version='8', arch='ia64', installtype='iso',
            expect_success=False)
    runtest(distro='RHEL-6', version='8', arch='x86_64', installtype='foo',
            expect_success=False)

def test_rhel7():
    for distro in ["RHEL-7", "CentOS-7"]:
        for version in ["0", "1", "2"]:
            for arch in ["i386", "x86_64"]:
                for installtype in ["url", "iso"]:
                    runtest(distro=distro, version=version, arch=arch,
                            installtype=installtype, expect_success=True)
    runtest(distro='RHEL-7', version='U9', arch='x86_64', installtype='url',
            expect_success=False)
    runtest(distro='RHEL-7', version='2', arch='ia64', installtype='iso',
            expect_success=False)
    runtest(distro='RHEL-7', version='2', arch='x86_64', installtype='foo',
            expect_success=False)

def test_debian():
    for version in ["5", "6", "7", "8", "9"]:
        for arch in ["i386", "x86_64"]:
            runtest(distro='Debian', version=version, arch=arch,
                    installtype='iso', expect_success=True)
    runtest(distro='Debian', version='U9', arch='x86_64', installtype='url',
            expect_success=False)
    runtest(distro='Debian', version='9', arch='ia64', installtype='iso',
            expect_success=False)
    runtest(distro='Debian', version='9', arch='x86_64', installtype='foo',
            expect_success=False)

def test_windows():
    runtest(distro='Windows', version='2000', arch='i386', installtype='iso',
            expect_success=True)
    for version in ["XP", "2003", "2008", "7", "8", "2012", "8.1", "2016", "10"]:
        for arch in ["i386", "x86_64"]:
            runtest(distro='Windows', version=version, arch=arch, installtype='iso',
                    expect_success=True)
    runtest(distro='Windows', version='U9', arch='x86_64', installtype='iso',
            expect_success=False)
    runtest(distro='Windows', version='2012', arch='ia64', installtype='iso',
            expect_success=False)
    runtest(distro='Windows', version='2012', arch='x86_64', installtype='foo',
            expect_success=False)
    runtest(distro='Windows', version='2000', arch='x86_64', installtype='iso',
            expect_success=False)

def test_opensuse():
    for version in ["10.3", "11.0", "11.1", "11.2", "11.3", "11.4", "12.1",
                    "12.2", "12.3", "13.1", "13.2"]:
        for arch in ["i386", "x86_64"]:
            runtest(distro='OpenSUSE', version=version, arch=arch, installtype='iso',
                    expect_success=True)
    runtest(distro='OpenSUSE', version='U9', arch='x86_64', installtype='iso',
            expect_success=False)
    runtest(distro='OpenSUSE', version='13.2', arch='ia64', installtype='iso',
            expect_success=False)
    runtest(distro='OpenSUSE', version='13.2', arch='x86_64', installtype='foo',
            expect_success=False)

def test_ubuntu():
    for version in ["5.04", "5.10", "6.06", "6.06.1", "6.06.2", "6.10", "7.04",
                    "7.10", "8.04", "8.04.1", "8.04.2", "8.04.3", "8.04.4",
                    "8.10", "9.04", "9.10", "10.04", "10.04.1", "10.04.2",
                    "10.04.3", "10.10", "11.04", "11.10", "12.04", "12.04.1",
                    "12.04.2", "12.04.3", "12.10", "13.04", "13.10", "14.04",
                    "14.10", "15.04", "15.10", "16.04", "16.10", "17.04",
                    "17.10", "18.04"]:
        for arch in ["i386", "x86_64"]:
            for installtype in ["iso", "url"]:
                runtest(distro='Ubuntu', version=version, arch=arch,
                        installtype=installtype, expect_success=True)
    runtest(distro='Ubuntu', version='U9', arch='x86_64', installtype='iso',
            expect_success=False)
    runtest(distro='Ubuntu', version='16.04', arch='ia64', installtype='iso',
            expect_success=False)
    runtest(distro='Ubuntu', version='16.04', arch='x86_64', installtype='foo',
            expect_success=False)

def test_mandrake():
    for version in ["8.2", "9.1", "9.2", "10.0", "10.1"]:
        runtest(distro='Mandrake', version=version, arch='i386',
                installtype='iso', expect_success=True)
    runtest(distro='Mandrake', version='U9', arch='x86_64', installtype='iso',
            expect_success=False)
    runtest(distro='Mandrake', version='10.1', arch='ia64', installtype='iso',
            expect_success=False)
    runtest(distro='Mandrake', version='10.1', arch='x86_64', installtype='foo',
            expect_success=False)
    runtest(distro='Mandrake', version='10.1', arch='x86_64', installtype='url',
            expect_success=False)

def test_mandriva():
    for version in ["2005", "2006.0", "2007.0", "2008.0"]:
        for arch in ["i386", "x86_64"]:
            runtest(distro='Mandriva', version=version, arch=arch,
                    installtype='iso', expect_success=True)
    runtest(distro='Mandriva', version='U9', arch='x86_64', installtype='iso',
            expect_success=False)
    runtest(distro='Mandriva', version='2008.0', arch='ia64', installtype='iso',
            expect_success=False)
    runtest(distro='Mandriva', version='2008.0', arch='x86_64', installtype='foo',
            expect_success=False)
    runtest(distro='Mandriva', version='2008.0', arch='x86_64', installtype='url',
            expect_success=False)

def test_mageia():
    for version in ["2", "3", "4", "4.1", "5"]:
        for arch in ["i386", "x86_64"]:
            runtest(distro='Mageia', version=version, arch=arch,
                    installtype='iso', expect_success=True)
    runtest(distro='Mageia', version='U9', arch='x86_64', installtype='iso',
            expect_success=False)
    runtest(distro='Mageia', version='5', arch='ia64', installtype='iso',
            expect_success=False)
    runtest(distro='Mageia', version='5', arch='x86_64', installtype='foo',
            expect_success=False)



# Copyright (C) 2010,2011  Chris Lalancette <clalance@redhat.com>
# Copyright (C) 2012-2017  Chris Lalancette <clalancette@gmail.com>

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation;
# version 2.1 of the License.

# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

"""
RHL installation
"""

import os
import re
import shutil

import oz.OzException
import oz.RedHat
import oz.ozutil


class RHL9Guest(oz.RedHat.RedHatLinuxCDGuest):
    """
    Class for RHL-9 installation.
    """
    def __init__(self, tdl, config, auto, output_disk, netdev, diskbus,
                 macaddress):
        # RHL-9 doesn't support direct kernel/initrd booting; it hangs right
        # after unpacking the initrd
        oz.RedHat.RedHatLinuxCDGuest.__init__(self, tdl, config, auto,
                                              output_disk, netdev, diskbus,
                                              False, True, None, macaddress)

        if self.tdl.arch != "i386":
            raise oz.OzException.OzException("Invalid arch " + self.tdl.arch + "for RHL guest")

    def _modify_iso(self):
        """
        Method to modify the ISO for autoinstallation.
        """
        self.log.debug("Putting the kickstart in place")

        outname = os.path.join(self.iso_contents, "ks.cfg")

        if self.default_auto_file():
            def _kssub(line):
                """
                Method that is called back from oz.ozutil.copy_modify_file() to
                modify kickstart files as appropriate for RHL-9.
                """
                # because we need to do this URL substitution here, we can't use
                # the generic "copy_kickstart()" method
                if re.match("^url", line):
                    return "url --url " + self.url + "\n"
                elif re.match("^rootpw", line):
                    return "rootpw " + self.rootpw + "\n"
                return line

            oz.ozutil.copy_modify_file(self.auto, outname, _kssub)
        else:
            shutil.copy(self.auto, outname)

        initrdline = "  append initrd=initrd.img ks=cdrom:/ks.cfg method=" + self.url
        self._modify_isolinux(initrdline)

    def get_auto_path(self):
        """
        Method to create the correct path to the RHL kickstart files.
        """
        return oz.ozutil.generate_full_auto_path("RedHatLinux" + self.tdl.update + ".auto")


class RHL7xand8Guest(oz.RedHat.RedHatFDGuest):
    """
    Class for RHL 7.0, 7.1, 7.2, and 8 installation.
    """
    def __init__(self, tdl, config, auto, output_disk, nicmodel, diskbus,
                 macaddress):
        oz.RedHat.RedHatFDGuest.__init__(self, tdl, config, auto, output_disk,
                                         nicmodel, diskbus, macaddress)

    def get_auto_path(self):
        return oz.ozutil.generate_full_auto_path("RedHatLinux" + self.tdl.update + ".auto")


def get_class(tdl, config, auto, output_disk=None, netdev=None, diskbus=None,
              macaddress=None):
    """
    Factory method for RHL installs.
    """
    if tdl.update in ["9"]:
        return RHL9Guest(tdl, config, auto, output_disk, netdev, diskbus,
                         macaddress)

    if tdl.update in ["7.2", "7.3", "8"]:
        return RHL7xand8Guest(tdl, config, auto, output_disk, netdev, diskbus,
                              macaddress)

    if tdl.update in ["7.0", "7.1"]:
        if netdev is None:
            netdev = "ne2k_pci"
        return RHL7xand8Guest(tdl, config, auto, output_disk, netdev, diskbus,
                              macaddress)


def get_supported_string():
    """
    Return supported versions as a string.
    """
    return "RHL: 7.0, 7.1, 7.2, 7.3, 8, 9"



# Copyright (C) 2013-2017  Chris Lalancette <clalancette@gmail.com>
# Copyright (C) 2013       Ian McLeod <imcleod@redhat.com>

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation;
# version 2.1 of the License.

# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

"""
RHEL-7 installation
"""

import os

import oz.OzException
import oz.RedHat
import oz.ozutil


class RHEL7Guest(oz.RedHat.RedHatLinuxCDYumGuest):
    """
    Class for RHEL-7 installation
    """
    def __init__(self, tdl, config, auto, output_disk=None, netdev=None,
                 diskbus=None, macaddress=None):
        oz.RedHat.RedHatLinuxCDYumGuest.__init__(self, tdl, config, auto,
                                                 output_disk, netdev, diskbus,
                                                 True, True, "cpio", macaddress,
                                                 True)
        self.virtio_channel_name = 'org.fedoraproject.anaconda.log.0'

    def _modify_iso(self):
        """
        Method to modify the ISO for autoinstallation.
        """
        self._copy_kickstart(os.path.join(self.iso_contents, "ks.cfg"))

        initrdline = "  append initrd=initrd.img ks=cdrom:/dev/cdrom:/ks.cfg"
        if self.tdl.installtype == "url":
            initrdline += " repo=" + self.url
        else:
            # RHEL6 dropped this command line directive due to an Anaconda bug
            # that has since been fixed.  Note that this used to be "method="
            # but that has been deprecated for some time.
            initrdline += " repo=cdrom:/dev/cdrom"
        self._modify_isolinux(initrdline)

    def get_auto_path(self):
        """
        Method to create the correct path to the RHEL 7 kickstart file.
        """
        return oz.ozutil.generate_full_auto_path("RHEL7.auto")


def get_class(tdl, config, auto, output_disk=None, netdev=None, diskbus=None,
              macaddress=None):
    """
    Factory method for RHEL-7 installs.
    """
    if tdl.update.isdigit():
        if netdev is None:
            netdev = 'virtio'
        if diskbus is None:
            diskbus = 'virtio'
        return RHEL7Guest(tdl, config, auto, output_disk, netdev, diskbus,
                          macaddress)


def get_supported_string():
    """
    Return supported versions as a string.
    """
    return "RHEL 7"



# Copyright (C) 2010,2011  Chris Lalancette <clalance@redhat.com>
# Copyright (C) 2012-2018  Chris Lalancette <clalancette@gmail.com>

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation;
# version 2.1 of the License.

# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

"""
OpenSUSE installation
"""

import os
import re
import shutil

import lxml.etree

import oz.GuestFSManager
import oz.Linux
import oz.OzException
import oz.ozutil


class OpenSUSEConfiguration(object):
    """
    Configuration class for OpenSUSE.
    """
    def __init__(self, reboots, use_floppy_autoyast, extended_zypper_list,
                 default_netdev, default_diskbus):
        self._reboots = reboots
        self._use_floppy_autoyast = use_floppy_autoyast
        self._extended_zypper_list = extended_zypper_list
        self._default_netdev = default_netdev
        self._default_diskbus = default_diskbus

    @property
    def reboots(self):
        """
        Property method for the number of reboots for installation of this
        version of OpenSUSE.
        """
        return self._reboots

    @property
    def use_floppy_autoyast(self):
        """
        Property method to determine whether the autoyast should go on a
        floppy or on the ISO.
        """
        return self._use_floppy_autoyast

    @property
    def extended_zypper_list(self):
        """
        Property method for where the URI is stored in the output of
        'zypper list' in this version of OpenSUSE.
        """
        return self._extended_zypper_list

    @property
    def default_netdev(self):
        """
        Property method for the default netdev for this version of OpenSUSE.
        """
        return self._default_netdev

    @property
    def default_diskbus(self):
        """
        Property method for the default diskbus for this version of OpenSUSE.
        """
        return self._default_diskbus


version_to_config = {
    "42.2": OpenSUSEConfiguration(reboots=1, use_floppy_autoyast=False,
                                  extended_zypper_list=True, default_netdev='virtio',
                                  default_diskbus='virtio'),
    "42.1": OpenSUSEConfiguration(reboots=1, use_floppy_autoyast=False,
                                  extended_zypper_list=True, default_netdev='virtio',
                                  default_diskbus='virtio'),
    "13.2": OpenSUSEConfiguration(reboots=1, use_floppy_autoyast=True,
                                  extended_zypper_list=False, default_netdev='virtio',
                                  default_diskbus='virtio'),
    "13.1": OpenSUSEConfiguration(reboots=1, use_floppy_autoyast=False,
                                  extended_zypper_list=False, default_netdev='virtio',
                                  default_diskbus='virtio'),
    "12.3": OpenSUSEConfiguration(reboots=1, use_floppy_autoyast=False,
                                  extended_zypper_list=False, default_netdev='virtio',
                                  default_diskbus='virtio'),
    "12.2": OpenSUSEConfiguration(reboots=1, use_floppy_autoyast=False,
                                  extended_zypper_list=False, default_netdev='virtio',
                                  default_diskbus='virtio'),
    "12.1": OpenSUSEConfiguration(reboots=1, use_floppy_autoyast=False,
                                  extended_zypper_list=False, default_netdev='virtio',
                                  default_diskbus='virtio'),
    "11.4": OpenSUSEConfiguration(reboots=1, use_floppy_autoyast=False,
                                  extended_zypper_list=False, default_netdev='virtio',
                                  default_diskbus='virtio'),
    "11.3": OpenSUSEConfiguration(reboots=1, use_floppy_autoyast=False,
                                  extended_zypper_list=False, default_netdev='virtio',
                                  default_diskbus='virtio'),
    "11.2": OpenSUSEConfiguration(reboots=1, use_floppy_autoyast=False,
                                  extended_zypper_list=False, default_netdev='virtio',
                                  default_diskbus='virtio'),
    "11.1": OpenSUSEConfiguration(reboots=1, use_floppy_autoyast=False,
                                  extended_zypper_list=False, default_netdev='virtio',
                                  default_diskbus='virtio'),
    "11.0": OpenSUSEConfiguration(reboots=1, use_floppy_autoyast=False,
                                  extended_zypper_list=False, default_netdev='virtio',
                                  default_diskbus='virtio'),
    "10.3": OpenSUSEConfiguration(reboots=0, use_floppy_autoyast=False,
                                  extended_zypper_list=False, default_netdev=None,
                                  default_diskbus=None),
}


class OpenSUSEGuest(oz.Linux.LinuxCDGuest):
    """
    Class for OpenSUSE installation.
    """
    def __init__(self, tdl, config, auto, output_disk, nicmodel, diskbus,
                 macaddress):
        self.config = version_to_config[tdl.update]
        if nicmodel is None:
            nicmodel = self.config.default_netdev
        if diskbus is None:
            diskbus = self.config.default_diskbus
        oz.Linux.LinuxCDGuest.__init__(self, tdl, config, auto, output_disk,
                                       nicmodel, diskbus, True, False,
                                       macaddress)

        self.crond_was_active = False
        self.sshd_was_active = False

    def _modify_iso(self):
        """
        Method to make the boot ISO auto-boot with appropriate parameters.
        """
        self.log.debug("Putting the autoyast in place")

        outname = os.path.join(self.iso_contents, "autoinst.xml")

        if self.default_auto_file():
            doc = lxml.etree.parse(self.auto)

            pw = doc.xpath('/suse:profile/suse:users/suse:user/suse:user_password',
                           namespaces={'suse': 'http://www.suse.com/1.0/yast2ns'})
            if len(pw) != 1:
                raise oz.OzException.OzException("Invalid SUSE autoyast file; expected single user_password, saw %d" % (len(pw)))
            pw[0].text = self.rootpw

            with open(outname, 'w') as f:
                f.write(lxml.etree.tostring(doc, pretty_print=True, encoding="unicode"))
        else:
            shutil.copy(self.auto, outname)

        if self.config.use_floppy_autoyast:
            oz.ozutil.subprocess_check_output(["/sbin/mkfs.msdos", "-C",
                                               self.output_floppy, "1440"])
            oz.ozutil.subprocess_check_output(["mcopy", "-n", "-o", "-i",
                                               self.output_floppy, outname,
                                               "::autoinst.xml"])

        self.log.debug("Modifying the boot options")
        isolinux_cfg = os.path.join(self.iso_contents, "boot", self.tdl.arch,
                                    "loader", "isolinux.cfg")
        f = open(isolinux_cfg, "r")
        lines = f.readlines()
        f.close()
        for index, line in enumerate(lines):
            if re.match("timeout", line):
                lines[index] = "timeout 1\n"
            elif re.match("default", line):
                lines[index] = "default customiso\n"
        lines.append("label customiso\n")
        lines.append("  kernel linux\n")
        if self.config.use_floppy_autoyast:
            # OpenSUSE 13.2 has an utterly broken autoyast that can't figure out
            # how to read the autoinst.xml from the CD.  Use a floppy instead.
            lines.append("  append initrd=initrd splash=silent autoyast=floppy:///autoinst.xml")
        else:
            lines.append("  append initrd=initrd splash=silent instmode=cd autoyast=default")

        with open(isolinux_cfg, 'w') as f:
            f.writelines(lines)

    def _generate_new_iso(self):
        """
        Method to create a new ISO based on the modified CD/DVD.
        """
        self.log.info("Generating new ISO")
        oz.ozutil.subprocess_check_output(["genisoimage", "-r", "-V", "Custom",
                                           "-J", "-no-emul-boot",
                                           "-b", "boot/" + self.tdl.arch + "/loader/isolinux.bin",
                                           "-c", "boot/" + self.tdl.arch + "/loader/boot.cat",
                                           "-boot-load-size", "4",
                                           "-boot-info-table", "-graft-points",
                                           "-iso-level", "4", "-pad",
                                           "-allow-leading-dots", "-l", "-v",
                                           "-o", self.output_iso,
                                           self.iso_contents],
                                          printfn=self.log.debug)

    def install(self, timeout=None, force=False):
        """
        Method to run the operating system installation.
        """
        extrainstalldevs = None
        if self.config.use_floppy_autoyast:
            extrainstalldevs = [self._InstallDev("floppy", self.output_floppy, "fda")]
        return self._do_install(timeout, force, self.config.reboots, None, None, None, extrainstalldevs)

    def _image_ssh_teardown_step_1(self, g_handle):
        """
        First step to undo _image_ssh_setup (remove authorized keys).
        """
        self.log.debug("Teardown step 1")
        # reset the authorized keys
        self.log.debug("Resetting authorized_keys")
        g_handle.path_restore('/root/.ssh/authorized_keys')

    def _image_ssh_teardown_step_2(self, g_handle):
        """
        Second step to undo _image_ssh_setup (remove custom sshd_config).
        """
        self.log.debug("Teardown step 2")
        # remove custom sshd_config
        self.log.debug("Resetting sshd_config")
        g_handle.path_restore('/etc/ssh/sshd_config')

        # reset the service link
        self.log.debug("Resetting sshd service")
        if g_handle.exists('/usr/lib/systemd/system/sshd.service'):
            if not self.sshd_was_active:
                g_handle.remove_if_exists('/etc/systemd/system/multi-user.target.wants/sshd.service')
        else:
            g_handle.path_restore('/etc/init.d/after.local')

    def _image_ssh_teardown_step_3(self, g_handle):
        """
        Third step to undo _image_ssh_setup (remove guest announcement).
        """
        self.log.debug("Teardown step 3")
        # remove announce cronjob
        self.log.debug("Resetting announcement to host")
        g_handle.remove_if_exists('/etc/NetworkManager/dispatcher.d/99-reportip')
        g_handle.remove_if_exists('/etc/cron.d/announce')

        # remove reportip
        self.log.debug("Removing reportip")
        g_handle.remove_if_exists('/root/reportip')

        # reset the service link
        self.log.debug("Resetting cron service")
        if g_handle.exists('/usr/lib/systemd/system/cron.service'):
            if not self.crond_was_active:
                g_handle.rm('/etc/systemd/system/multi-user.target.wants/cron.service')
        else:
            runlevel = self.get_default_runlevel(g_handle)
            startuplink = '/etc/rc.d/rc' + runlevel + ".d/S06cron"
            g_handle.path_restore(startuplink)

    def _image_ssh_teardown_step_4(self, g_handle):
        """
        Fourth step to undo changes by the operating system.  For instance,
        during first boot openssh generates ssh host keys and stores them
        in /etc/ssh.  Since this image might be cached later on, this method
        removes those keys.
        """
        for f in ["/etc/ssh/ssh_host_dsa_key", "/etc/ssh/ssh_host_dsa_key.pub",
                  "/etc/ssh/ssh_host_rsa_key", "/etc/ssh/ssh_host_rsa_key.pub",
                  "/etc/ssh/ssh_host_ecdsa_key", "/etc/ssh/ssh_host_ecdsa_key.pub",
                  "/etc/ssh/ssh_host_key", "/etc/ssh/ssh_host_key.pub"]:
            g_handle.remove_if_exists(f)

        # Remove any lease files; this is so that subsequent boots don't try
        # to connect to a DHCP server that is on a totally different network
        for lease in g_handle.glob_expand("/var/lib/dhcp/*.leases"):
            g_handle.rm(lease)

        for lease in g_handle.glob_expand("/var/lib/dhcp6/*.leases"):
            g_handle.rm(lease)

        for lease in g_handle.glob_expand("/var/lib/dhcp6/*.lease"):
            g_handle.rm(lease)

    def _collect_teardown(self, libvirt_xml):
        """
        Method to reverse the changes done in _collect_setup.
        """
        self.log.info("Collection Teardown")

        g_handle = oz.GuestFSManager.GuestFSLibvirtFactory(libvirt_xml, self.libvirt_conn)
        g_handle.mount_partitions()

        try:
            self._image_ssh_teardown_step_1(g_handle)

            self._image_ssh_teardown_step_2(g_handle)

            self._image_ssh_teardown_step_3(g_handle)

            self._image_ssh_teardown_step_4(g_handle)
        finally:
            g_handle.cleanup()
            shutil.rmtree(self.icicle_tmp)

    def do_icicle(self, guestaddr):
        """
        Method to collect the package information and generate the ICICLE
        XML.
        """
        self.log.debug("Generating ICICLE")
        stdout, stderr_unused, retcode_unused = self.guest_execute_command(guestaddr,
                                                                           'rpm -qa',
                                                                           timeout=30)

        return self._output_icicle_xml(stdout.split("\n"),
                                       self.tdl.description)

    def _image_ssh_setup_step_1(self, g_handle):
        """
        First step for allowing remote access (generate and upload ssh keys).
        """
        # part 1; upload the keys
        self.log.debug("Step 1: Uploading ssh keys")
        if not g_handle.exists('/root/.ssh'):
            g_handle.mkdir('/root/.ssh')

        g_handle.path_backup('/root/.ssh/authorized_keys')

        self._generate_openssh_key(self.sshprivkey)

        g_handle.upload(self.sshprivkey + ".pub", '/root/.ssh/authorized_keys')

    def _image_ssh_setup_step_2(self, g_handle):
        """
        Second step for allowing remote access (ensure sshd is running).
        """
        # part 2; check and setup sshd
        self.log.debug("Step 2: setup sshd")

        if not g_handle.exists('/usr/sbin/sshd'):
            raise oz.OzException.OzException("ssh not installed on the image, cannot continue")

        if g_handle.exists('/usr/lib/systemd/system/sshd.service'):
            if g_handle.exists('/etc/systemd/system/multi-user.target.wants/sshd.service'):
                self.sshd_was_active = True
            else:
                g_handle.ln_sf('/usr/lib/systemd/system/sshd.service',
                               '/etc/systemd/system/multi-user.target.wants/sshd.service')
        else:
            g_handle.path_backup("/etc/init.d/after.local")
            local = os.path.join(self.icicle_tmp, "after.local")
            with open(local, "w") as f:
                f.write("/sbin/service sshd start\n")
            try:
                g_handle.upload(local, "/etc/init.d/after.local")
            finally:
                os.unlink(local)

        sshd_config_file = self.icicle_tmp + "/sshd_config"
        with open(sshd_config_file, 'w') as f:
            f.write("""PasswordAuthentication no
UsePAM yes

X11Forwarding yes

Subsystem      sftp    /usr/lib64/ssh/sftp-server

AcceptEnv LANG LC_CTYPE LC_NUMERIC LC_TIME LC_COLLATE LC_MONETARY LC_MESSAGES
AcceptEnv LC_PAPER LC_NAME LC_ADDRESS LC_TELEPHONE LC_MEASUREMENT
AcceptEnv LC_IDENTIFICATION LC_ALL
""")

        try:
            g_handle.path_backup("/etc/ssh/sshd_config")
            g_handle.upload(sshd_config_file, '/etc/ssh/sshd_config')
        finally:
            os.unlink(sshd_config_file)

    def _image_ssh_setup_step_3(self, g_handle):
        """
        Third step for allowing remote access (make the guest announce itself
        on bootup).
        """
        # part 3; make sure the guest announces itself
        self.log.debug("Step 3: Guest announcement")

        scriptfile = os.path.join(self.icicle_tmp, "script")

        if g_handle.exists("/etc/NetworkManager/dispatcher.d"):
            with open(scriptfile, 'w') as f:
                f.write("""\
#!/bin/bash

if [ "$1" = "eth0" -a "$2" = "up" ]; then
    echo -n "!$DHCP4_IP_ADDRESS,%s!" > /dev/ttyS1
fi
""" % (self.uuid))

            try:
                g_handle.upload(scriptfile,
                                '/etc/NetworkManager/dispatcher.d/99-reportip')
                g_handle.chmod(0o755,
                               '/etc/NetworkManager/dispatcher.d/99-reportip')
            finally:
                os.unlink(scriptfile)

        if not g_handle.exists('/usr/sbin/cron'):
            raise oz.OzException.OzException("cron not installed on the image, cannot continue")

        with open(scriptfile, 'w') as f:
            f.write("""\
#!/bin/bash
DEV=$(/bin/awk '{if ($2 == 0) print $1}' /proc/net/route) &&
[ -z "$DEV" ] && exit 0
ADDR=$(/sbin/ip -4 -o addr show dev $DEV | /bin/awk '{print $4}' | /usr/bin/cut -d/ -f1) &&
[ -z "$ADDR" ] && exit 0
echo -n "!$ADDR,%s!" > /dev/ttyS1
""" % (self.uuid))

        try:
            g_handle.upload(scriptfile, '/root/reportip')
            g_handle.chmod(0o755, '/root/reportip')
        finally:
            os.unlink(scriptfile)

        announcefile = os.path.join(self.icicle_tmp, "announce")
        with open(announcefile, 'w') as f:
            f.write('*/1 * * * * root /bin/bash -c "/root/reportip"\n')

        try:
            g_handle.upload(announcefile, '/etc/cron.d/announce')
        finally:
            os.unlink(announcefile)

        if g_handle.exists('/usr/lib/systemd/system/cron.service'):
            if g_handle.exists('/etc/systemd/system/multi-user.target.wants/cron.service'):
                self.crond_was_active = True
            else:
                g_handle.ln_sf('/lib/systemd/system/cron.service',
                               '/etc/systemd/system/multi-user.target.wants/cron.service')
        else:
            runlevel = self.get_default_runlevel(g_handle)
            startuplink = '/etc/rc.d/rc' + runlevel + ".d/S06cron"
            g_handle.path_backup(startuplink)
            g_handle.ln_sf('/etc/init.d/cron', startuplink)

    def _collect_setup(self, libvirt_xml):
        """
        Setup the guest for remote access.
        """
        self.log.info("Collection Setup")

        g_handle = oz.GuestFSManager.GuestFSLibvirtFactory(libvirt_xml, self.libvirt_conn)
        g_handle.mount_partitions()

        # we have to do 3 things to make sure we can ssh into OpenSUSE:
        # 1)  Upload our ssh key
        # 2)  Make sure sshd is running on boot
        # 3)  Make the guest announce itself to the host

        try:
            try:
                self._image_ssh_setup_step_1(g_handle)

                try:
                    self._image_ssh_setup_step_2(g_handle)

                    try:
                        self._image_ssh_setup_step_3(g_handle)
                    except:
                        self._image_ssh_teardown_step_3(g_handle)
                        raise
                except:
                    self._image_ssh_teardown_step_2(g_handle)
                    raise
            except:
                self._image_ssh_teardown_step_1(g_handle)
                raise

        finally:
            g_handle.cleanup()

    def _customize_repos(self, guestaddr):
        """
        Method to add user-provided repositories to the guest.
        """
        self.log.debug("Installing additional repository files")
        for repo in list(self.tdl.repositories.values()):
            self.guest_execute_command(guestaddr,
                                       "zypper addrepo %s %s" % (repo.url,
                                                                 repo.name))

    def _install_packages(self, guestaddr, packstr):
        # due to a bug in OpenSUSE 11.1, we want to remove the default
        # CD repo first
        stdout, stderr_unused, retcode_unused = self.guest_execute_command(guestaddr,
                                                                           'zypper repos -d')
        removerepos = []
        for line in stdout.split('\n'):
            if re.match("^[0-9]+", line):
                split = line.split('|')

                column = 7
                if self.config.extended_zypper_list:
                    # OpenSUSE Leap has the URI in the 8th column
                    column = 8
                if re.match("^cd://", split[column].strip()):
                    removerepos.append(split[0].strip())

        for repo in removerepos:
            self.guest_execute_command(guestaddr,
                                       'zypper removerepo %s' % (repo))

        self.guest_execute_command(guestaddr,
                                   'zypper -n install %s' % (packstr))

    def _remove_repos(self, guestaddr):
        for repo in list(self.tdl.repositories.values()):
            if not repo.persisted:
                self.guest_execute_command(guestaddr,
                                           "zypper removerepo %s" % (repo.name))


def get_class(tdl, config, auto, output_disk=None, netdev=None, diskbus=None,
              macaddress=None):
    """
    Factory method for OpenSUSE installs.
    """
    if tdl.update in version_to_config.keys():
        return OpenSUSEGuest(tdl, config, auto, output_disk, netdev, diskbus,
                             macaddress)


def get_supported_string():
    """
    Return supported versions as a string.
    """
    return "OpenSUSE: " + ", ".join(sorted(version_to_config.keys()))



# Copyright (C) 2011  Chris Lalancette <clalance@redhat.com>
# Copyright (C) 2012-2018  Chris Lalancette <clalancette@gmail.com>

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation;
# version 2.1 of the License.

# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

"""
Mandrake installation
"""

import os
import re
import shutil

import oz.Guest
import oz.OzException
import oz.ozutil


class MandrakeConfiguration(object):
    """
    Configuration class for Mandrake installation.
    """
    def __init__(self, old_isolinux, need_longer_timeout):
        self._old_isolinux = old_isolinux
        self._need_longer_timeout = need_longer_timeout

    @property
    def old_isolinux(self):
        """
        Property method for whether this version of Mandrake uses an old isolinux.
        """
        return self._old_isolinux

    @property
    def need_longer_timeout(self):
        """
        Property method for whether this version of Mandrake needs a longer timeout.
        """
        return self._need_longer_timeout


version_to_config = {
    '10.1': MandrakeConfiguration(old_isolinux=False, need_longer_timeout=False),
    '10.0': MandrakeConfiguration(old_isolinux=False, need_longer_timeout=False),
    '9.2': MandrakeConfiguration(old_isolinux=False, need_longer_timeout=False),
    '9.1': MandrakeConfiguration(old_isolinux=False, need_longer_timeout=False),
    '9.0': MandrakeConfiguration(old_isolinux=False, need_longer_timeout=True),
    '8.2': MandrakeConfiguration(old_isolinux=True, need_longer_timeout=True),
}


class MandrakeGuest(oz.Guest.CDGuest):
    """
    Class for Mandrake 8.2, 9.1, 9.2, 10.0, and 10.1 installation.
    """
    def __init__(self, tdl, config, auto, output_disk, netdev, diskbus,
                 macaddress):
        oz.Guest.CDGuest.__init__(self, tdl, config, auto, output_disk, netdev,
                                  None, None, diskbus, True, False, macaddress)

        if self.tdl.arch != "i386":
            raise oz.OzException.OzException("Mandrake only supports i386 architecture")

        self.config = version_to_config[tdl.update]

    def _modify_iso(self):
        """
        Method to make the boot ISO auto-boot with appropriate parameters.
        """
        self.log.debug("Modifying ISO")

        self.log.debug("Copying cfg file")

        outname = os.path.join(self.iso_contents, "auto_inst.cfg")

        if self.default_auto_file():

            def _cfg_sub(line):
                """
                Method that is called back from oz.ozutil.copy_modify_file() to
                modify preseed files as appropriate for Mandrake.
                """
                if re.search("'password' =>", line):
                    return "			'password' => '" + self.rootpw + "',\n"
                return line

            oz.ozutil.copy_modify_file(self.auto, outname, _cfg_sub)
        else:
            shutil.copy(self.auto, outname)

        self.log.debug("Modifying isolinux.cfg")
        if self.config.old_isolinux:
            syslinux = os.path.join(self.icicle_tmp, 'syslinux.cfg')
            with open(syslinux, 'w') as f:
                f.write("""\
default customiso
timeout 1
prompt 0
label customiso
  kernel vmlinuz
  append initrd=cdrom.rdz ramdisk_size=32000 root=/dev/ram3 automatic=method:cdrom vga=788 auto_install=auto_inst.cfg
    """)
            cdromimg = os.path.join(self.iso_contents, "Boot", "cdrom.img")
            oz.ozutil.subprocess_check_output(["mcopy", "-n", "-o", "-i",
                                               cdromimg, syslinux,
                                               "::SYSLINUX.CFG"],
                                              printfn=self.log.debug)
        else:
            isolinuxcfg = os.path.join(self.iso_contents, "isolinux",
                                       "isolinux.cfg")
            with open(isolinuxcfg, 'w') as f:
                f.write("""\
default customiso
timeout 1
prompt 0
label customiso
  kernel alt0/vmlinuz
  append initrd=alt0/all.rdz ramdisk_size=128000 root=/dev/ram3 acpi=ht vga=788 automatic=method:cdrom kickstart=auto_inst.cfg
    """)

    def _generate_new_iso(self):
        """
        Method to create a new ISO based on the modified CD/DVD.
        """
        self.log.info("Generating new ISO")
        if self.config.old_isolinux:
            oz.ozutil.subprocess_check_output(["genisoimage", "-r", "-V", "Custom",
                                               "-J", "-cache-inodes",
                                               "-b", "Boot/cdrom.img",
                                               "-c", "Boot/boot.cat",
                                               "-v", "-o", self.output_iso,
                                               self.iso_contents],
                                              printfn=self.log.debug)
        else:
            oz.ozutil.subprocess_check_output(["genisoimage", "-r", "-V", "Custom",
                                               "-J", "-l", "-no-emul-boot",
                                               "-b", "isolinux/isolinux.bin",
                                               "-c", "isolinux/boot.cat",
                                               "-boot-load-size", "4",
                                               "-cache-inodes", "-boot-info-table",
                                               "-v", "-o", self.output_iso,
                                               self.iso_contents],
                                              printfn=self.log.debug)

    def install(self, timeout=None, force=False):
        internal_timeout = timeout
        if internal_timeout is None and self.config.need_longer_timeout:
            internal_timeout = 2500
        return self._do_install(internal_timeout, force, 0)


def get_class(tdl, config, auto, output_disk=None, netdev=None, diskbus=None,
              macaddress=None):
    """
    Factory method for Mandrake installs.
    """
    if tdl.update in version_to_config.keys():
        return MandrakeGuest(tdl, config, auto, output_disk, netdev, diskbus,
                             macaddress)


def get_supported_string():
    """
    Return supported versions as a string.
    """
    return "Mandrake: " + ", ".join(sorted(version_to_config.keys(), key=float))



#!/usr/bin/python

import sys
import os

try:
    import lxml.etree
except ImportError:
    print('Unable to import lxml.  Is python-lxml installed?')
    sys.exit(1)

try:
    import py.test
except ImportError:
    print('Unable to import py.test.  Is py.test installed?')
    sys.exit(1)

# Find oz
prefix = '.'
for i in range(0,3):
    if os.path.isdir(os.path.join(prefix, 'oz')):
        sys.path.insert(0, prefix)
        break
    else:
        prefix = '../' + prefix

try:
    import oz
    import oz.TDL
except ImportError:
    print('Unable to import oz.  Is oz installed?')
    sys.exit(1)

# the tests dictionary lists all of the test we will run.  The key for the
# dictionary is the filename of the test, and the value is whether the test
# is expected to succeed (True) or not (False)
tests = {
    "test-01-simple-iso.tdl": True,
    "test-02-simple-url.tdl": True,
    "test-03-empty-template.tdl": False,
    "test-04-no-os.tdl": False,
    "test-05-no-name.tdl": False,
    "test-06-simple-iso-description.tdl": True,
    "test-07-packages-no-package.tdl": True,
    "test-08-repositories-no-repository.tdl": True,
    "test-09-os-invalid-arch.tdl": False,
    "test-10-os-invalid-install-type.tdl": False,
    "test-11-description-packages-repositories.tdl": True,
    "test-12-os-no-name.tdl": False,
    "test-13-os-no-version.tdl": False,
    "test-14-os-no-arch.tdl": False,
    "test-15-os-no-install.tdl": False,
    "test-16-signed-repository.tdl": True,
    "test-17-repo-invalid-signed.tdl": False,
    "test-18-rootpw.tdl": True,
    "test-19-key.tdl": True,
    "test-20-multiple-install.tdl": False,
    "test-21-missing-install-type.tdl": False,
    "test-22-md5sum.tdl": True,
    "test-23-sha1sum.tdl": True,
    "test-24-sha256sum.tdl": True,
    "test-25-md5sum-and-sha1sum.tdl": False,
    "test-26-md5sum-and-sha256sum.tdl": False,
    "test-27-sha1sum-and-sha256sum.tdl": False,
    "test-28-package-no-name.tdl": False,
    "test-29-files.tdl": True,
    "test-30-file-no-name.tdl": False,
    "test-31-file-raw-type.tdl": True,
    "test-32-file-base64-type.tdl": True,
    "test-33-file-invalid-type.tdl": False,
    "test-34-file-invalid-base64.tdl": False,
    "test-35-repository-no-name.tdl": False,
    "test-36-repository-no-url.tdl": False,
    "test-37-command.tdl": True,
    "test-38-command-no-name.tdl": False,
    "test-39-command-raw-type.tdl": True,
    "test-40-command-base64-type.tdl": True,
    "test-41-command-bogus-base64.tdl": False,
    "test-42-command-bogus-type.tdl": False,
    "test-43-persisted-repos.tdl": True,
    "test-44-version.tdl": True,
    "test-45-bogus-version.tdl": False,
    "test-46-duplicate-name.tdl": False,
    "test-47-invalid-template.tdl": False,
    "test-48-file-empty-base64.tdl": True,
    "test-49-file-empty-raw.tdl": True,
    "test-50-command-base64-empty.tdl": False,
    "test-51-disk-size.tdl": True,
    "test-52-command-file-url.tdl": True,
    "test-53-command-http-url.tdl": True,
    "test-54-files-file-url.tdl": True,
    "test-55-files-http-url.tdl": True,
    "test-56-invalid-disk-size.tdl": False,
    "test-57-invalid-disk-size.tdl": False,
    "test-58-disk-size-terabyte.tdl": True,
    "test-59-command-sorting.tdl": True,
}

# Test that iterates over all .tdl files
def test_all():
    for (tdl, expected_pass) in list(tests.items()):

        # locate full path for tdl file
        tdl_prefix = ''
        for tdl_prefix in ['tests/tdl/', 'tdl/', '']:
            if os.path.isfile(tdl_prefix + tdl):
                break
        tdl_file = tdl_prefix + tdl
        test_name = os.path.splitext(tdl,)[0]

        # Test if the TDL class will parse it.
        print("Testing %s" % (tdl_file))
        with open(tdl_file, 'r') as infp:
            try:
                oz.TDL.TDL(infp.read())
                if not expected_pass:
                    assert(False)
            except Exception:
                if expected_pass:
                    raise

'''
def test_persisted(tdl='test-43-persisted-repos.tdl'):
    # locate full path for tdl file
    tdl_prefix = ''
    for tdl_prefix in ['tests/tdl/', 'tdl/', '']:
        if os.path.isfile(tdl_prefix + tdl):
            break
    if not os.path.isfile(tdl_prefix + tdl):
        raise Exception('Unable to locate TDL: %s' % tdl)
    tdl_file = tdl_prefix + tdl
    test_name = os.path.splitext(tdl,)[0]

    # Grab TDL object
    tdl = validate_ozlib(tdl_file)

    def assert_persisted_value(persisted, value):
        assert persisted == value, \
            "expected %s, got %s" % (value, persisted)

    for repo in list(tdl.repositories.values()):
        if repo.name.endswith('true'):
            yield '%s_%s' % (test_name, repo.name), assert_persisted_value, repo.persisted, True
        else:
            yield '%s_%s' % (test_name, repo.name), assert_persisted_value, repo.persisted, False
'''



from distutils.core import setup, Command
from distutils.command.sdist import sdist as _sdist
import subprocess
import time

VERSION = '0.17.0'
RELEASE = '1'

datafiles = [('share/man/man1', ['man/oz-install.1', 'man/oz-generate-icicle.1',
                                 'man/oz-customize.1', 'man/oz-examples.1',
                                 'man/oz-cleanup-cache.1'])
             ]

class sdist(_sdist):
    """ custom sdist command, to prep oz.spec file for inclusion """

    def run(self):
        global VERSION
        global RELEASE

        # Create a development release string for later use
        git_head = subprocess.Popen("git log -1 --pretty=format:%h",
                                    shell=True,
                                    stdout=subprocess.PIPE).communicate()[0].strip()
        date = time.strftime("%Y%m%d%H%M%S", time.gmtime())
        git_release = "%sgit%s" % (date, git_head)

        # Expand macros in oz.spec.in and create oz.spec
        spec_in = open('oz.spec.in', 'r')
        spec = open('oz.spec', 'w')
        for line in spec_in.xreadlines():
            if "@VERSION@" in line:
                line = line.replace("@VERSION@", VERSION)
            elif "@RELEASE@" in line:
                # If development release, include date+githash in %{release}
                if RELEASE.startswith('0'):
                    RELEASE += '.' + git_release
                line = line.replace("@RELEASE@", RELEASE)
            spec.write(line)
        spec_in.close()
        spec.close()

        # Run parent constructor
        _sdist.run(self)

class pytest(Command):
    user_options = []
    def initialize_options(self): pass
    def finalize_options(self): pass
    def run(self):
        try:
            errno = subprocess.call('py.test-3 tests --verbose --tb=short --junitxml=tests/results.xml'.split())
        except OSError as e:
            if e.errno == 2:
                raise OSError(2, "No such file or directory: py.test")
            raise
        raise SystemExit(errno)

setup(name='oz',
      version=VERSION,
      description='Oz automated installer',
      author='Chris Lalancette',
      author_email='clalancette@gmail.com',
      license='LGPLv2',
      url='http://github.com/clalancette/oz',
      package_dir={'oz': 'oz'},
      package_data={'oz': ['auto/*', '*.rng']},
      packages=['oz'],
      scripts=['oz-install', 'oz-generate-icicle', 'oz-customize',
               'oz-cleanup-cache'],
      cmdclass={'sdist': sdist,
                'test' : pytest },
      data_files = datafiles,
      )
